# -*- coding: utf-8 -*-

# Copyright (c) 2008, Rectorate of the University of Freiburg
# Copyright (c) 2009,  Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""Provides unittest classes TestMRA and TestMRADiscontinuousDiscretisation.
"""



import unittest


import numpy
from pyphant.core import DataContainer as DC


class TestEmd5Source(unittest.TestCase):
    """Sets up a random FieldContainer and ragisters it with the
    knowledge manager."""
    def setUp(self):
        self.V = DC.FieldContainer(numpy.random.randn(10,10))
        self.V.seal()
        from pyphant.core import KnowledgeManager
        KnowledgeManager.KnowledgeManager.getInstance(
            ).registerDataContainer(self.V, temporary=True)

    def testEmd5Source(self):
        """Retrieves the previously registered FieldContainer via the
        Emd5Source and checks for equality."""
        #Predict result
        from pyphant.core.Emd5Src import Emd5Src
        s = Emd5Src()
        s.paramEmd5.value = self.V.id
        result = s.plugGetDataContainer.getResult()
        self.assertEqual(result, self.V)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

# Copyright (c) 2008-2009, Rectorate of the University of Freiburg
# Copyright (c) 2009-2010, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import zipfile
import numpy
import re
import copy
import StringIO
import codecs
from pyphant.core import DataContainer
from pyphant.quantities import (Quantity, isQuantity)
from pyphant.quantities.ParseQuantities import (parseQuantity, parseVariable,
                                                parseDateTime, str2unit)
import logging
_logger = logging.getLogger("pyphant")

def normation(normationStr):
    try:
        unit = Quantity(str(normationStr))
    except:
        try:
            unit = Quantity(1.0, str(normationStr))
        except:
            unit = float(normationStr)
    return unit

def loadDataFromZip(filename, subscriber=1):
    z = zipfile.ZipFile(filename, 'r')
    names = z.namelist()
    names.sort()
    total = len(names)
    assert total > 0, "The loaded FMF archive named %s \
                        does not contain any files." % filename
    data = {}
    for i, pixelName in enumerate(names):
        b = z.read(pixelName)
        rawContainer = readSingleFile(b, pixelName)
        if len(rawContainer) == 1:
            data[pixelName] = rawContainer[0]
        else:
            data[pixelName] = rawContainer
        subscriber %= float(i + 1) / total * 100.0
    return data, names

def collectAttributes(data, names):
    """Function collectAttributes(data)
    data: dictionary referencing the FMF attributes by the respective filenames
    returns tuple (dictionary of common attributes, dictionary of variable
    attributes).
    """
    #Collect attributes, define filename as new attribute
    atts = {u'filename': names}
    for filename in names:
        sc = data[filename]
        for section, sectionDict in sc.attributes.iteritems():
            for key, treetoken in sectionDict.iteritems():
                attlist = atts.setdefault(key, [])
                attlist.append(treetoken)
    #Separate common attributes from variable attributes
    commonAttr = {}
    variableAttr = {}
    for k, l in atts.iteritems():
        v = l[0]
        isConst = True
        for i in l[1:]:
            if i != v:
                isConst = False
                #break
        if isConst:
            commonAttr[k] = v
        else:
            variableAttr[k] = l
    return (commonAttr, variableAttr)


class column2Field:
    def __init__(self):
        self.Np = 0
        self.Nt = 0

    def norm(self, datum, unit, error=False):
        if isQuantity(datum):
            try:
                return datum.inUnitsOf(unit).value
            except:
                raise ValueError, "The datum %s cannot be expressed \
                                    in terms of %s." % (datum, unit)
        elif error:
            return 0.0
        else:
            return numpy.NaN

    def __call__(self, longname, column):
        tupples = filter(lambda c: type(c) == type((0,)), column)
        hasTupples = len(tupples) > 0
        if hasTupples:
            tuppleLength = max(map(len, tupples))
        if hasTupples:
            if tuppleLength == 2:
                indexDatum = 0
                indexError = 1
                if isQuantity(tupples[0][1]) and tupples[0][1].isCompatible('s'):
                    shortname = 't_%i' % self.Nt
                    self.Nt += 1
                else:
                    shortname = 'p_%i' % self.Np
                    self.Np += 1
                for i, element in enumerate(column):
                    if not type(element) == type((0,)):
                        column[i] = (numpy.NaN, None)
            elif tuppleLength == 3:
                shortname = tupples[0][0]
                indexDatum = 1
                indexError = 2
                for i, element in enumerate(column):
                    if not type(element) == type((0,)):
                        column[i] = (shortname, numpy.NaN, None)
            try:
                data = [element[indexDatum] for element in column]
            except:
                print longname, column
                import sys
                sys.exit(0)
            error = [element[indexError] for element in column]
            unitCandidates = [element.unit for element in data \
                              if isQuantity(element)]
            if len(unitCandidates) == 0:
                unit = 1.0
            else:
                unit = unitCandidates[0]
            normation = lambda arg: self.norm(arg, unit)
            field = numpy.array(map(normation, data))
            ErrorNormation = lambda arg: self.norm(arg, unit, error=True)
            result = DataContainer.FieldContainer(field,
                                error=numpy.array(map(ErrorNormation, error)),
                                mask=numpy.isnan(field),
                                unit=Quantity(1.0, unit),
                                shortname=shortname,
                                longname=longname)
        else:
            #Joining lists of strings
            if type(column[0]) == type([]):
                firstElement = column[0][0]
            else:
                firstElement = column[0]
            if type(firstElement) in (type(''), type(u'')):
                for i in xrange(len(column)):
                    if type(column[i]) == type([]):
                        column[i] = ','.join(column[i])
            result = DataContainer.FieldContainer(numpy.array(column),
                                                  longname=longname)
        return result
column2FieldContainer = column2Field()

def unpackAndCollateFields(variableAttr, data):
    fieldData = {}
    dependencies = {}
    units = {}
    shortnames = {}
    for filename in variableAttr['filename']:
        sample = data[filename]
        for field in sample.columns:
            if field.longname in fieldData:
                conversionFactor = normation(field.unit) / units[field.longname]
                fieldData[field.longname].append(field.data * conversionFactor)
                dimensionNames = [dim.longname for dim in field.dimensions
                                  if dim.longname != u'Index']
                if len(dimensionNames) > 1:
                    _logger.warning("Is specified, but has not been tested!")
                if dependencies[field.longname] != dimensionNames:
                    raise ValueError, "The data sets of the FMF archive have \
                                        inconsistent dependencies in section \
                                        [*data definitions]."
            else:
                fieldData[field.longname] = [field.data]
                dependencies[field.longname] = [dim.longname for dim
                                                in field.dimensions
                                                if dim.longname != u'Index']
                units[field.longname] = normation(field.unit)
                shortnames[field.longname] = field.shortname
    return fieldData, dependencies, units, shortnames

def checkAndCondense(data):
    reference = data[0]
    for element in data[1:]:
        numpy.testing.assert_array_almost_equal(reference, element)
    return reference

def readZipFile(filename, subscriber=1):
    data, names = loadDataFromZip(filename, subscriber)
    commonAttr, variableAttr = collectAttributes(data, names)
    #Wrap variable attributes into FieldContainer
    containers = [column2FieldContainer(longname, column) for longname, column
                  in variableAttr.iteritems()]
    #Process SampleContainers of parsed FMF files and skip independent
    #variables, which are used as dimensions.
    fieldData, dependencies, units, shortnames = unpackAndCollateFields(
        variableAttr, data)
    independentFieldsNames = []
    for fieldName, dependency in dependencies.iteritems():
        if dependencies[fieldName] == []:
            independentFieldsNames.append(fieldName)
    for fieldName in independentFieldsNames:
        del dependencies[fieldName]
    #Build independent fields
    independentFields = {}
    for indepField in independentFieldsNames:
        indepData = checkAndCondense(fieldData[indepField])
        independentFields[indepField] = DataContainer.FieldContainer(
                                            numpy.array(indepData),
                                            longname=indepField,
                                            shortname=shortnames[indepField],
                                            unit=units[indepField],
                                            rescale=True
                                            )
    #Build dependent fields
    #QUESTION: Can a field depend on a dependent field?
    for field, dependency in dependencies.iteritems():
        newField = DataContainer.FieldContainer(numpy.array(fieldData[field]),
                                                longname=field,
                                                shortname=shortnames[field],
                                                unit=units[field],
                                                rescale=True)
        for i, indepField in enumerate(dependency):
            dim = len(newField.dimensions) - i - 1
            newField.dimensions[dim] = independentFields[indepField]
        assert newField.isValid()
        containers.append(newField)
    #The next lines are a hack and should be dealt with properly...
    if u'creator' in commonAttr.keys():
        creator = commonAttr[u'creator']
        del commonAttr[u'creator']
        result = DataContainer.SampleContainer(containers,
                                               attributes=commonAttr)
        result.creator = creator
    else:
        result = DataContainer.SampleContainer(containers,
                                               attributes=commonAttr)
    return result

def reshapeField(field):
    if field.isIndependent() or len(field.dimensions) == 1:
        return field
    dimData = [numpy.unique(d.data) for d in field.dimensions]
    dimDicts = [dict([(data, index) for index, data in enumerate(dimdata)])
                for dimdata in dimData]
    fieldData = numpy.ones([len(d) for d in dimData]) * numpy.nan
    indicess = zip(*[map(lambda x: dimDicts[index][x], dim.data) \
                     for index, dim in enumerate(field.dimensions)])
    for datum, indices in zip(field.data, indicess):
        fieldData[indices] = datum
    newDims = [ DataContainer.FieldContainer(dimData[i],
                                             f.unit,
                                             longname=f.longname,
                                             shortname=f.shortname,
                                             attributes=f.attributes)
                for i, f in enumerate(field.dimensions) ]
    newField = DataContainer.FieldContainer(fieldData,
                                            field.unit,
                                            mask=numpy.isnan(fieldData),
                                            dimensions=newDims,
                                            longname=field.longname,
                                            shortname=field.shortname,
                                            attributes=field.attributes)
    return newField

def readDataFile(filename):
    filehandle = open(filename, 'r')
    dat = filehandle.read()
    filehandle.close()
    rawContainer = readSingleFile(dat, filename)
    if len(rawContainer) == 1:
        container = rawContainer[0]
        container.seal()
        return container
    newSample = DataContainer.SampleContainer(rawContainer,
                        longname='List of tables',
                        shortname='L',
                        attributes=copy.deepcopy(rawContainer[0].attributes))
    newSample.seal()
    return newSample

def loadFMFFromFile(filename, subscriber=0):
    try:
        return readZipFile(filename, subscriber=subscriber)
    except zipfile.BadZipfile:
        return readDataFile(filename)

def readSingleFile(b, pixelName):
    _logger.info(u"Parsing file %s." % pixelName)
    preParsedData, d, FMFversion, commentChar = preParseData(b)
    if commentChar == "#":
        commentChar = "\#"
    from configobj import ConfigObj, ConfigObjError
    class FMFConfigObj(ConfigObj):
        #   ConfigObj sets the following default
        #   in opposition to FMF
        #   Function            ConfigObj   FMF
        #   Key-Value-Seperator "="         ":"
        #   CommentChar         "#"         "#",";", ....
        #
        #   So we have to redefine our regexp
        #
        _keyword = re.compile(r'''^ # line start
            (\s*)                   # indentation
            (                       # keyword
                (?:".*?")|          # double quotes
                (?:'.*?')|          # 'single quotes
                (?:[^'":].*?)       # no quotes
            )
            \s*:\s*                 # divider
            (.*)                    # value (including list values and comments)
            $   # line end
            ''', re.VERBOSE)  #'

        _sectionmarker = re.compile(r'''^
            (\s*)                     # 1: indentation
            ((?:\[\s*)+)              # 2: section marker open
            (                         # 3: section name open
                (?:"\s*\S.*?\s*")|    # at least one non-space with double quotes
                (?:'\s*\S.*?\s*')|    # at least one non-space with single quotes
                (?:[^'"\s].*?)        # at least one non-space unquoted
            )                         # section name close
            ((?:\s*\])+)              # 4: section marker close
            \s*(%s.*)?                # 5: optional comment
            $''' % commentChar,
            re.VERBOSE)

        _valueexp = re.compile(r'''^
            (?:
                (?:
                    (
                        (?:
                            (?:
                                (?:".*?")|              # double quotes
                                (?:'.*?')|              # single quotes
                                (?:[^'",%s][^,%s]*?)    # unquoted
                            )
                            \s*,\s*                     # comma
                        )*      # match all list items ending in a comma (if any)
                    )
                    (
                        (?:".*?")|                      # double quotes
                        (?:'.*?')|                      # single quotes
                        (?:[^'",%s\s][^,]*?)|  # unquoted
                        (?:(?<!,))                      # Empty value
                    )?          # last item in a list - or string value
                )|
                (,)             # alternatively a single comma - empty list
            )
            \s*(%s.*)?          # optional comment
            $''' % ((commentChar, )*4),
            re.VERBOSE)

        # use findall to get the members of a list value
        _listvalueexp = re.compile(r'''
            (
                (?:".*?")|          # double quotes
                (?:'.*?')|          # single quotes
                (?:[^'",%s]?.*?)       # unquoted
            )
            \s*,\s*                 # comma
            ''' % commentChar,
            re.VERBOSE)

        # this regexp is used for the value
        # when lists are switched off
        _nolistvalue = re.compile(r'''^
            (
                (?:".*?")|          # double quotes
                (?:'.*?')|          # single quotes
                (?:[^'"%s].*?)|     # unquoted
                (?:)                # Empty value
            )
            \s*(%s.*)?              # optional comment
            $''' % (commentChar, commentChar),
            re.VERBOSE)

        # regexes for finding triple quoted values on one line
        _single_line_single = re.compile(
                                r"^'''(.*?)'''\s*(%s.*)?$" % commentChar,
                                re.VERBOSE)
        _single_line_double = re.compile(
                                r'^"""(.*?)"""\s*(%s.*)?$' % commentChar,
                                re.VERBOSE)
        _multi_line_single = re.compile(
                                r"^(.*?)'''\s*(%s.*)?$" % commentChar,
                                re.VERBOSE)
        _multi_line_double = re.compile(
                                r'^(.*?)"""\s*(%s.*)?$' % commentChar,
                                re.VERBOSE)
    try:
        config = FMFConfigObj(d.encode('utf-8').splitlines(), encoding='utf-8')
    except ConfigObjError, e:
        from sys import exit
        exit('%s\nPlease check the syntax of the FMF-file, in particular \
                the correct usage of comments.' % e)
    return config2tables(preParsedData, config, FMFversion)

def parseBool(value):
    if value.lower() == 'true':
        return True
    elif value.lower() == 'false':
        return False
    raise AttributeError

def getConverters(FMFversion='1.1'):
    converters = [
        int,
        float,
        parseBool,
        lambda v: parseVariable(v,FMFversion),
        lambda q: parseQuantity(q,FMFversion),
        complex,        # Complex is checked after variables and quantities,
                        # because 1J is 1 Joule and not an imaginary number.
        lambda d: parseDateTime(d,FMFversion),
        ]
    return converters

def item2value(oldVal, FMFversion='1.1'):
    if type(oldVal) == type([]):
        for c in getConverters(FMFversion):
            try:
                return map(c, oldVal)
            except:
                pass
    for c in getConverters(FMFversion):
        try:
            return c(oldVal)
        except:
            pass
    return oldVal

def config2tables(preParsedData, config, FMFversion='1.1'):
    if config.has_key('*table definitions'):
        longnames = dict([(i, k) for k, i
                          in config['*table definitions'].iteritems()])
        del config['*table definitions']
    else:
        longnames = { None : 'Table' }
    tables = []
    for k in config:
        if k.startswith('*data definitions'):
            if k == '*data definitions':
                shortname = None
            else:
                shortname = k.split(':')[1].strip()
            tables.append(data2table(longnames[shortname],
                                     shortname,
                                     preParsedData[shortname],
                                     config[k],FMFversion))
            del config[k]
    attributes = config.walk(lambda section, key:
                             item2value(section[key], FMFversion))
    for t in tables:
        t.attributes = copy.deepcopy(attributes)
    return tables

def data2table(longname, shortname, preParsedData, config, FMFversion='1.1'):
    datTable = []
    shapelen = len(preParsedData.shape)
    if len(config) == 1:
        if shapelen == 0:
            preParsedData = preParsedData.reshape((1, 1))
        elif shapelen == 1:
            preParsedData = preParsedData.reshape((1, preParsedData.shape[0]))
    else:
        if shapelen == 1:
            preParsedData = preParsedData.reshape((preParsedData.shape[0], 1))
    assert len(preParsedData.shape) == 2
    for col in preParsedData:
        try:
            result = col.astype('i')
        except ValueError, e:
            try:
                result = col.astype('f')
            except ValueError, e:
                try:
                    result = col.astype('complex')
                except ValueError, e:
                    result = col
        datTable.append(result)
    colspec_re = re.compile(ur"(?P<shortname>[^\s([]*)\s*(?P<deps>\([^)]*\))?\s*(?:(?:\\pm|\+-|\+/-)\s*(?P<error>[^\s[]*))?\s*(?P<unit>\[[^]]*])?")
    fields = []
    fields_by_name = {}
    dimensions_for_fields = {}
    errors_for_fields = {}
    for i, (fieldLongname, spec) in enumerate(config.items()):
        if type(spec) == type([]):
            spec = ','.join(spec)
        try:
            match = re.search(colspec_re, spec)
        except TypeError, e:
            _logger.error("""Cannot interpret definition of data column "%s", \
                            which is given as "%s"!""" % (fieldLongname, spec))
        unit = match.group('unit')
        if unit != None:
            unit = str2unit(unit[1: -1], FMFversion)
        else:
            unit = 1.0
        fieldShortname = match.group('shortname')
        dimensions = match.group('deps')
        if dimensions != None:
            dimensions_for_fields[fieldShortname] = dimensions[1: -1].split(',')
        else:
            dimensions_for_fields[fieldShortname] = None
        errors_for_fields[fieldShortname] = match.group('error')
        field = DataContainer.FieldContainer(datTable[i],
                                             longname=fieldLongname,
                                             shortname=fieldShortname,
                                             unit=unit)
        fields.append(field)
        fields_by_name[fieldShortname] = field
    for f in fields:
        sn = f.shortname
        dims = dimensions_for_fields[sn]
        if dims != None:
            f.dimensions = [fields_by_name[d] for d in dims]
        error = errors_for_fields[sn]
        if error != None:
            try:
                error = float(error)
                f.error = numpy.ones(f.data.shape) * error
            except:
                f.error = fields_by_name[error].inUnitsOf(f).data
    reshapedFields = []
    for field in fields:
        try:
            newField = reshapeField(field)
        except TypeError:
            raise
            if field.data.dtype.name.startswith('string'):
                _logger.warning('Warning: Cannot reshape numpy.array \
                                   of string: %s' % field)
                newField = field
            else:
                _logger.error('Error: Cannot reshape numpy.array: %s' % field)
                import sys
                sys.exit(0)
        reshapedFields.append(newField)
    if shortname == None:
        shortname = 'T'
    return DataContainer.SampleContainer(reshapedFields,
                                         longname=longname,
                                         shortname=shortname)

def preParseData(b):
    localVar = {'fmf-version':'1.1', 'coding':'utf-8', 'delimiter':'\t'}
    commentChar = ';'
    if b.startswith(codecs.BOM_UTF8):
        b = b.lstrip(codecs.BOM_UTF8)
    if b[0] == ';' or b[0] == '#':
        commentChar = b[0]
        items =  [var.strip().split(':') for var
                  in b.split('-*-')[1].split(';')]
        try:
            for key, value in items:
                localVar[key.strip()] = value.strip()
                if localVar[key.strip()] == 'whitespace':
                    localVar[key.strip()] = None
                if localVar[key.strip()] == 'semicolon':
                    localVar[key.strip()] = ';'
        except ValueError, e:
            from sys import exit
            exit('%s\nPlease, check syntax of headline, presumably a key \
                    and its value are not separated by a colon.' % e)
    d = unicode(b, localVar['coding'])
    dataExpr = re.compile(ur"^(\[\*data(?::\s*([^\]]*))?\]\r?\n)([^[]*)",
                          re.MULTILINE | re.DOTALL)
    commentExpr = re.compile(ur"^%s.*"%commentChar, re.MULTILINE)
    d = re.sub(commentExpr, '', d)
    preParsedData = {}
    def preParseData(match):
        try:
            preParsedData[match.group(2)] = numpy.loadtxt(
                                            StringIO.StringIO(match.group(3)),
                                            unpack=True,
                                            comments=commentChar,
                                            dtype='S',
                                            delimiter=localVar['delimiter']
                                            )
        except Exception:
            return match.group(0)
        return u""
    d = re.sub(dataExpr, preParseData, d)
    return preParsedData, d, str(localVar['fmf-version']), commentChar



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2009, Rectorate of the University of Freiburg
# Copyright (c) 2009, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""


import pkg_resources
try:
    pkg_resources.require('matplotlib>=0.98.1')
    MPL_LT_0_98_1 = False
except:
    MPL_LT_0_98_1 = True

import pylab, scipy, numpy
from pyphant.core.Connectors import TYPE_IMAGE
from pyphant.wxgui2.DataVisReg import DataVisReg
from pyphant.quantities import isQuantity
#from NonUniformImage import NonUniformImage
from matplotlib.image import NonUniformImage


class ImageVisualizer(object):
    name='Image Visualizer'
    def __init__(self, fieldContainer,show=True):
        self.fieldContainer = fieldContainer
        self.show = show
        self.execute()

    def dataPrinter(self,event):
        x=self.fieldContainer.dimensions[-1]
        y=self.fieldContainer.dimensions[-2]
        zLabel=self.fieldContainer.shortname
        if event.inaxes:
            xc=event.xdata
            yc=event.ydata
            xi = numpy.abs(x.data-xc).argmin()
            yi = numpy.abs(y.data-yc).argmin()
            if ((self.fieldContainer.mask != None)
                and self.fieldContainer.mask[yi, xi]):
                val = "n/a"
            else:
                try:
                    val = self.fieldContainer.data[yi, xi]
                    val *= self.fieldContainer.unit
                except IndexError:
                    val = "nan"
            xval = xc * x.unit
            yval = yc * y.unit
            def format(val):
                if not isQuantity(val):
                    if type(val) in (type(' '),type(u' ')):
                        valstr = val
                    else:
                        valstr = "%.4g" % val
                else:
                    valstr = "%.3f %s" % (val.value,val.unit.name())
                return valstr
            labels = map(format,[xval,yval,val])
            labels.insert(0,zLabel)
            self.figure.canvas.toolbar.set_message("%s(%s,%s) = %s"
                                                   % tuple(labels))
        else:
            self.figure.canvas.toolbar.set_message("outside axis")

    def execute(self):
        pylab.ioff()
        self.figure = pylab.figure()
        self.figure.canvas.mpl_connect('motion_notify_event', self.dataPrinter)
        x = self.fieldContainer.dimensions[-1].data
        y = self.fieldContainer.dimensions[-2].data
        xmin=scipy.amin(x)
        xmax=scipy.amax(x)
        ymin=scipy.amin(y)
        ymax=scipy.amax(y)
        #Support for images with non uniform axes adapted
        #from python-matplotlib-doc/examples/pcolor_nonuniform.py
        ax = self.figure.add_subplot(111)
        vmin = self.fieldContainer.attributes.get('vmin', None)
        vmax = self.fieldContainer.attributes.get('vmax', None)
        if vmin is not None:
            vmin /= self.fieldContainer.unit
        if vmax is not None:
            vmax /= self.fieldContainer.unit
        if MPL_LT_0_98_1 or self.fieldContainer.isLinearlyDiscretised():
            pylab.imshow(self.fieldContainer.maskedData,
                         aspect='auto',
                         interpolation='nearest',
                         vmin=vmin,
                         vmax=vmax,
                         origin='lower',
                         extent=(xmin, xmax, ymin, ymax))
            pylab.colorbar(ax=ax)
        else:
            im = NonUniformImage(ax, extent=(xmin,xmax,ymin,ymax))
            if vmin is not None or vmax is not None:
                im.set_clim(vmin, vmax)
                im.set_data(x, y, self.fieldContainer.maskedData)
            else:
                im.set_data(x, y, self.fieldContainer.maskedData)
                im.autoscale_None()
            ax.images.append(im)
            ax.set_xlim(xmin,xmax)
            ax.set_ylim(ymin,ymax)
            pylab.colorbar(im, ax=ax)
        pylab.xlabel(self.fieldContainer.dimensions[-1].shortlabel)
        pylab.ylabel(self.fieldContainer.dimensions[-2].shortlabel)
        pylab.title(self.fieldContainer.label)
        #ax=pylab.gca()
        if self.show:
            pylab.ion()
            pylab.show()


DataVisReg.getInstance().registerVisualizer(TYPE_IMAGE, ImageVisualizer)



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2007, Rectorate of the University of Freiburg
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""


import wx, os.path

class FileButton(wx.Button):
    def __init__(self, parent, param, validator):
        self.path=os.path.abspath(param.value)
        self.dir, self.filename=os.path.split(self.path)
        wx.Button.__init__(self, parent,
                           label=self.filename,
                           validator=validator)
        self.Bind(wx.EVT_BUTTON, self.selectFile)
        self.param = param

    def selectFile(self, event):
        dir = filename = wx.EmptyString
        if os.path.exists(self.dir):
            dir = self.dir
            if os.path.exists(os.path.join(self.dir, self.filename)):
                filename = self.filename
        dlg = wx.FileDialog(self, message="Choose a file", defaultDir=dir,
                            defaultFile=filename, style=wx.OPEN)
        if hasattr(self.param, "fileMask"):
            dlg.SetWildcard(self.param.fileMask)
        if dlg.ShowModal() == wx.ID_OK:
            self.path=os.path.abspath(dlg.GetPath())
            self.dir, self.filename=os.path.split(self.path)
            self.SetLabel(self.filename)

    def getValue(self):
        return self.path

    def SetValue(self, path):
        self.path=path
        self.dir, self.filename=os.path.split(self.path)
        self.SetLabel(self.filename)



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2008, Rectorate of the University of Freiburg
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

def getPyphantPath(subdir = ''):
    """
    returns full pyphant path with optional subdirectory
    subdir -- subdirectory that is created if it does not exist already,
              recursive creation of directories is supported also.
    """
    import os
    homedir = os.path.expanduser('~')
    if homedir == '~':
        homedir = os.getcwdu()
    path = os.path.join(homedir, '.pyphant', subdir)
    # Create the dir in a multi process save way:
    try:
        os.makedirs(path) #<-- ignores EEXIST in all but top recursion levels
    except OSError, ose:
        if ose.errno != os.errno.EEXIST:
            raise
    return path

def getUsername():
    import getpass
    return getpass.getuser()

def getMachine():
    import socket
    return unicode(socket.getfqdn(), 'utf-8')

def enableLogging():
    """
    Enables logging to stdout for debug purposes.
    """
    import logging
    import sys
    l = logging.getLogger("pyphant")
    l.setLevel(logging.DEBUG)
    f = logging.Formatter('%(asctime)s [%(name)s|%(levelname)s] %(message)s')
    h = logging.StreamHandler(sys.stderr)
    h.setFormatter(f)
    l.addHandler(h)
    l.info("Logger 'pyphant' has been configured for debug purposes.")

def uc2utf8(stype):
    """
    Returns utf-8 encoded version of stype, if stype was unicode, else
    stype is returned.
    If stype is of ListType, the above applies for all entries of the list.
    """
    from types import UnicodeType, ListType
    def convert(arg):
        if isinstance(arg, UnicodeType):
            return arg.encode('utf-8')
        return arg
    if isinstance(stype, ListType):
        return map(convert, stype)
    return convert(stype)

def utf82uc(stype):
    """
    Returns a unicode object created from a utf-8 encoded string.
    If the input was unicode, it is returned unchanged.
    List are treated similar to uc2utf8, see docstring there.
    """
    from types import StringType, ListType
    def convert(arg):
        if isinstance(arg, StringType):
            return unicode(arg, 'utf')
        return arg
    if isinstance(stype, ListType):
        return map(convert, stype)
    return convert(stype)

def emd52dict(emd5):
    """
    returns a dictionary with keys
    ('machine', 'creator', 'date', 'hash', 'type')
    """
    emd5 = utf82uc(emd5)
    emd5_split = emd5.split('/')
    retdict = {}
    retdict['machine'] = emd5_split[2]
    retdict['creator'] = emd5_split[3]
    retdict['date'] = emd5_split[4]
    retdict['hash'] = emd5_split[5].split('.')[0]
    retdict['type'] = emd5_split[5].split('.')[1]
    return retdict

def batch(recipe, input, plug, longname, dobatch=True, temporary=False):
    """
    Runs the same recipe multiple times for different input data.
    The return value is either a SampleContainer similar to input
    with 'emd5' column replaced by results or the resulting
    DataContainer from plug, if dobatch is set to False.
    recipe -- CompositeWorker instance
    input -- SampleContainer with 'emd5' column or any DataContainer if
             dobatch is set to False
    plug -- plug contained in recipe to get output from
            (there has to be exactly one open socket in recipe
            ascending from plug)
    longname -- longname of resulting SampleContainer, works only for
                dobatch == True
    dobatch -- if set to False, input is treated as a single data source
    temporary -- whether to register results temporarily, only applies when
                 dobatch is set to True
    """
    socket = recipe.getOpenSocketsForPlug(plug)[0]
    from pyphant.core.Emd5Src import Emd5Src
    DummyWorker = Emd5Src()
    socket.insert(DummyWorker.getPlugs()[0])
    DummyWorker.paramSelectby.value = u"enter emd5"
    from pyphant.core.KnowledgeManager import KnowledgeManager
    km = KnowledgeManager.getInstance()
    if dobatch:
        import copy
        output = copy.deepcopy(input)
        index = 0
        for emd5 in input['emd5'].data:
            DummyWorker.paramEnteremd5.value = emd5
            resultDC = plug.getResult()
            km.registerDataContainer(resultDC, temporary=temporary)
            output['emd5'].data[index] = resultDC.id
            index += 1
        output.longname = longname
        output.seal()
    else:
        km.registerDataContainer(input)
        DummyWorker.paramEnteremd5.value = input.id
        output = plug.getResult()
    socket.pullPlug()
    return output

def makeSC(column_data, longnames, shortnames, longname, shortname,
           attributes={}):
    unzipped = zip(*column_data)
    assert len(unzipped) == len(longnames) == len(shortnames)
    def get_column_fc(col, ln, sn):
        try:
            from pyphant.quantities import Quantity
            unit = Quantity(1.0, col[0].unit)
            data = [quant.value for quant in col]
        except (KeyError, AttributeError):
            unit = 1
            data = col
        from numpy import array
        from pyphant.core.DataContainer import FieldContainer
        fc = FieldContainer(data=array(data), unit=unit,
                            longname=ln, shortname=sn)
        return fc
    columns = [get_column_fc(col, ln, sn) for col, ln, sn \
               in zip(unzipped, longnames, shortnames)]
    from pyphant.core.DataContainer import SampleContainer
    sc = SampleContainer(longname=longname, shortname=shortname,
                         attributes=attributes, columns=columns)
    sc.seal()
    return sc

from threading import Lock
TIMESTAMP_LOCK = Lock()
LAST_TIMESTAMP = ''
del Lock

def getModuleUniqueTimestamp():
    global TIMESTAMP_LOCK
    global LAST_TIMESTAMP
    TIMESTAMP_LOCK.acquire()
    timestamp = None
    try:
        from datetime import datetime
        while True:
            timestamp = datetime.utcnow()
            if timestamp != LAST_TIMESTAMP:
                LAST_TIMESTAMP = timestamp
                break
            else:
                from time import sleep
                sleep(.001)
    finally:
        TIMESTAMP_LOCK.release()
    return timestamp


def parseFCUnit(s):
    """
    Parses the string/unicode s into a float or a Quantity.

    Returns a float or a Quantity or raises an error.
    See quantities package for possible errors.
    """
    s = uc2utf8(s)
    try:
        return float(s)
    except ValueError:
        from pyphant.quantities import Quantity
        return Quantity(s)



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2007, Rectorate of the University of Freiburg
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""


import copy, threading, inspect, logging
import Queue

class FullSocketError(ValueError):
    pass

TYPE_ARRAY = 'scipy.array'
TYPE_IMAGE = 'pil.image'
TYPE_INT = type(2)
TYPE_STRING = type("")
SUBTYPE_FILE = "filename"
TYPE_INACTIVE = type(None)
TYPE_BOOL = type(True)
DEFAULT_DATA_TYPE = TYPE_ARRAY

class Connector(object):
    def __init__(self, worker, name, type=DEFAULT_DATA_TYPE, pre=""):
        self.worker = worker
        self.name = name
        self.type = type
        self._isExternal = True
        self.pre = pre

    def _getIsExternal(self):
        return self._isExternal

    def _setIsExternal(self, isExternal):
        if isExternal != self._isExternal:
            self._isExternal = isExternal
            self.worker.connectorsExternalizationStateChanged(self)
    isExternal = property(_getIsExternal, _setIsExternal)
    id = property(lambda self:
                  self.worker.id+"."+self.pre+self.name.capitalize())

class Computer(threading.Thread):
    def __init__(self, method, exception_queue, **kwargs):
        threading.Thread.__init__(self)
        self.method = method
        self.kwargs = kwargs
        self.result = None
        self.exception_queue = exception_queue

    def run(self):
        if self.method:
            try:
                self.result = self.method(subscriber=self.kwargs["subscriber"])
            except Exception, e:
                logging.getLogger('pyphant').error(
                    u"An unhandled exception occured in the calculation.",
                    exc_info = True)
                self.exception_queue.put(e)
                raise


class Plug(Connector):
    def __init__(self, worker, name, type=DEFAULT_DATA_TYPE):
        Connector.__init__(self, worker, name, type, "plug")
        self._result = None
        self._resultLock = threading.Lock()
        self._sockets = []

    def __getstate__(self):#this could be done with marshalling
        pdict = copy.copy(self.__dict__)
        pdict['_result'] = None
        del pdict['_resultLock']
        return pdict

    def __setstate__(self, pdict):
        self.__dict__.update(pdict)
        self._resultLock = threading.Lock()

    def addSocket(self, socket):
        self._sockets.append(socket)

    def removeSocket(self, socket):
        self._sockets.remove(socket)

    def invalidate(self, event=None):
        self._resultLock.acquire()
        self._result = None
        for socket in self._sockets:
            socket.invalidate()
        self._resultLock.release()

    def resultIsAvailable(self):
        """Indicates whether a precalculated result is available.

        This is not threadsafe, i.e. by the time this function returns
        a formerly available result may have been invalidated, or a
        calculation may have finished. Use as an indicator only."""
        return (self._result != None)


class Updater(object):
    def __init__(self, subscriber, process):
        self.subscriber = subscriber
        self.process = process

    def __imod__(self, percentage):
        if self.subscriber:
            self.subscriber.updateProcess(self.process, percentage)
        return self

class CalculatingPlug(Plug):
    def __init__(self, method, name, type=DEFAULT_DATA_TYPE):
        Plug.__init__(self, method.im_self, name, type)
        self._methodName = method.func_name
        self._func = self.createWrapper(method)

    def createWrapper(self, method):
        args, varargs, varkw, defaults = inspect.getargspec(method)
        sockets = args[1:-1]
        name = method.func_name+'PyphantWrapper'
        l = 'def '+name+'(subscriber, method=method, process=self):\n'
        l += '\texception_queue=Queue.Queue()\n'
        for s in sockets:
            l += '\t'+s+'=Computer(method.im_self.getSocket("'
            l += s+'").getResult, exception_queue, subscriber=subscriber)\n'
        for s in sockets:
            l += '\t'+s+'.start()\n'
        for s in sockets:
            l += '\t'+s+'.join()\n'
        l += '\texceptions=[]\n'
        l += '\twhile not exception_queue.empty():\n'
        l += '\t\texceptions.append(exception_queue.get())\n'
        l += '\t\texception_queue.task_done()\n'
        l += '\tif len(exceptions)>0:\n'
        l += '\t\traise RuntimeError, str(exceptions)\n'
        #If no sockets are needed the comma in the next line will be erased,
        #so do not add a space!
        l += '\treturn method( subscriber=Updater(subscriber, process),'
        for s in sockets:
            l += s+'='+s+'.result,'
        l = l[:-1]+')\n'
        exec l
        return eval(name)

    def __getstate__(self):#this could be done with marshalling
        pdict = super(CalculatingPlug, self).__getstate__()
        del pdict['_func']
        return pdict

    def __setstate__(self, pdict):
        super(CalculatingPlug, self).__setstate__(pdict)
        self._func = self.createWrapper(getattr(self.worker, self._methodName))

    def getResult(self, subscriber = None):
        self._resultLock.acquire()
        try:
            if not self.resultIsAvailable():
                if subscriber:
                    subscriber.startProcess(self)
                self._result = self._func(subscriber)
                if subscriber:
                    subscriber.finishProcess(self)
            result = self._result
        finally:
            self._resultLock.release()
        return result

class Socket(Connector):
    def __init__(self, worker, name, type=DEFAULT_DATA_TYPE):
        Connector.__init__(self, worker, name, type, "socket")
        self._plug = None

    def isFull(self):
        return self._plug != None

    def insert(self, plug):
        assert(isinstance(plug, Plug))
        if self.isFull():
            raise FullSocketError()
        self._plug = plug
        plug.addSocket(self)
        self.invalidate()
        try:
            self.worker.connectionCreated(plug, self)
        except AttributeError:
            ## WARNING: Next line is a hack! Check for parentship properly!
            if self.worker.parent:
                self.worker.parent.connectionCreated(plug, self)

    def pullPlug(self):
        self._plug.removeSocket(self)
        plug = self._plug
        self._plug = None
        self.invalidate()
        try:
            self.worker.connectionDestroyed(plug, self)
        except AttributeError:
            ## WARNING: Next line is a hack! Check for parentship properly!
            if self.worker.parent:
                self.worker.parent.connectionDestroyed(plug, self)

    def invalidate(self, event=None):
        self.worker.invalidate(self)

    def getPlug(self):
        return self._plug

    def getResult(self, subscriber = None):
        return self._plug.getResult(subscriber)
    value = property(getResult)

class ConnectorProxy(Socket, Plug):
    def __init__(self, worker, isSocket, name, type=DEFAULT_DATA_TYPE):
        Plug.__init__(self, worker, name, type)
        Socket.__init__(self, worker, name, type)
        self._result = None
        self.isSocket = isSocket
        self._resultLock = threading.Lock()
        self._sockets = []

    def __getstate__(self):#this could be done with marshalling
        pdict = copy.copy(self.__dict__)
        pdict['_result'] = None
        del pdict['_resultLock']
        return pdict

    def __setstate__(self, pdict):
        self.__dict__.update(pdict)
        self._resultLock = threading.Lock()

    def invalidate(self, event=None):
        #if self.isSocket:
        #    Socket.invalidate(self, event)
        #else:
        Plug.invalidate(self, event)

    def getResult(self, subscriber=None):
        return self._plug.getResult(subscriber)
    value = property(getResult)



# -*- coding: utf-8 -*-

# Copyright (c) 2013, Rectorate of the University of Freiburg
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import unittest
from pyphant.core import (Worker, Connectors, CompositeWorker)
from pyphant.core.H5FileHandler import H5FileHandler
import pkg_resources
import tempfile
import shutil
import os
import logging


class DummyProvider(Worker.Worker):
    API = 2
    VERSION = 1
    REVISION = pkg_resources.get_distribution("pyphant").version
    name = "DummyProvider"
    _params = [("source", u"Source", "", None)]

    @Worker.plug(Connectors.TYPE_IMAGE)
    def load(self, subscriber=0):
        if not self.paramSource.value:
            raise ValueError('Not initialized!')
        else:
            return [42, -1]

    def __hash__(self):
        return 2


class DummyConsumer(Worker.Worker):
    API = 2
    VERSION = 1
    REVISION = pkg_resources.get_distribution("pyphant").version
    name = "DummyConsumer"
    _params = [("foo", u"Foo", [0], None)]
    _sockets = [("socket1", Connectors.TYPE_IMAGE)]

    def refreshParams(self, subscriber=None):
        if self.socketSocket1.isFull():
            values = self.socketSocket1.getResult(subscriber)
            self.paramFoo.possibleValues = values

    @Worker.plug(Connectors.TYPE_IMAGE)
    def check(self, socket1, subscriber=0):
        return True

    def __hash__(self):
        return 1


class RaiseHandler(logging.Handler):
    def handle(self, record):
        if record.exc_info:
            raise record.exc_info[1]


class TestInitalizationOrder(unittest.TestCase):
    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()
        self.filename = os.path.join(self.tmpdir, 'init_order.h5')

    def tearDown(self):
        shutil.rmtree(self.tmpdir)

    def testInitializationOrder(self):
        logger = logging.getLogger("pyphant")
        loghandler = RaiseHandler()
        logger.addHandler(loghandler)
        recipe = CompositeWorker.CompositeWorker()
        recipe.addWorker(DummyConsumer())
        recipe.addWorker(DummyProvider())
        recipe.getWorker('DummyProvider').paramSource.value = 'The Universe'
        plug = recipe.getWorker('DummyProvider').getPlug('load')
        socket = recipe.getWorker('DummyConsumer').getSocket('socket1')
        socket.insert(plug)
        handler = H5FileHandler(self.filename, 'w')
        with handler:
            handler.saveRecipe(recipe)
        handler = H5FileHandler(self.filename, 'r')
        with handler:
            handler.loadRecipe()


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2009, Rectorate of the University of Freiburg
# Copyright (c) 2009, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""Provides unittest classes FieldContainerTestCase.
"""


import unittest

import scipy
import copy, datetime
from pyphant.quantities import Quantity
from pyphant.core.DataContainer import FieldContainer, SampleContainer
from pyphant.core.PyTablesPersister import (saveField, loadField, saveSample,
                                            loadSample, saveExecutionOrder,
                                            loadExecutionOrders)
import numpy
import tables

class ContainerTestCase(unittest.TestCase):
    def setUp(self):
        self.testData = scipy.array([[0.,1.,2.],[3.,4.,5.],[6.,7.,8.]])
        self.longname = u"Sampled Data"
        self.shortname = u"I\\omega"
        self.unit = Quantity('3.14 m')
        self.attributes = {
            'id': 'dummy id',
            'author': 'dummy author',
            'version': 'dummy version',
            'unit': self.unit
            }
        self.eln = tables.openFile('FieldContainerTestCase.h5','w',
                                   title='Testing the saving and restoring of FieldContainers.')
        self.eln.createGroup(self.eln.root,'results')
        self.field = FieldContainer(self.testData,longname=self.longname,
                                    shortname=self.shortname,
                                    unit = self.unit,
                                    attributes = self.attributes
                                    )


    def tearDown(self):
        self.eln.close()
        import os
        os.remove('FieldContainerTestCase.h5')

class FieldContainerTestCase(ContainerTestCase):
    def testSaveRestore(self):
        self.field.creator=u"Klaus"
        self.field.seal()
        self.eln.createGroup(self.eln.root,'testSaveRestoreField')
        saveField(self.eln,self.eln.root.testSaveRestoreField,self.field)
        restoredField = loadField(self.eln,self.eln.root.testSaveRestoreField)
        self.assertEqual(restoredField,self.field)

    def testUnicodeFields(self):
        self.field.seal()
        unicodeArray = numpy.array([u'Hallo World!',u'Hallo Wörld!'])
        unicodeField = FieldContainer(unicodeArray,longname=u"blabla",
                                    shortname=self.shortname,
                                    unit = 1,
                                    attributes = self.attributes
                                    )
        unicodeField.seal()
        self.eln.createGroup(self.eln.root,'testUnicodeFields')
        saveField(self.eln,self.eln.root.testUnicodeFields,unicodeField)
        restoredField = loadField(self.eln,self.eln.root.testUnicodeFields)
        self.assertEqual(restoredField,unicodeField,
                         "Restored unicode string is %s (%s) but is expected to be %s (%s)." % (restoredField.data,restoredField.data.dtype,unicodeField.data,unicodeField.data.dtype))

class ObjectArrayTestCase(ContainerTestCase):
    def testDateTime(self):
        """Test the correct saving and  restoring of object arrays composed from datetime objects."""
        objectArray = numpy.array([datetime.datetime.now() for i in range(10)])
        objectField = FieldContainer(objectArray,longname=u"timestamp",
                                    shortname='t')
        objectField.seal()
        self.eln.createGroup(self.eln.root,'testObjectFields')
        saveField(self.eln,self.eln.root.testObjectFields,objectField)
        restoredField = loadField(self.eln,self.eln.root.testObjectFields)
        for i,j in zip(restoredField.data.tolist(),objectField.data.tolist()):
            self.assertEqual(i,j,'Expected %s but got %s!' % (j,i))

class FieldContainerTestCaseWithError(FieldContainerTestCase):
    def setUp(self):
        super(FieldContainerTestCase,self).setUp()
        self.field.error = numpy.ones(self.field.data.shape,'float')

class FieldContainerTestCaseWithMask(FieldContainerTestCase):
    def setUp(self):
        super(FieldContainerTestCase,self).setUp()
        self.field.mask = self.field.data>3

class SampleContainerTestCase(ContainerTestCase):
    def setUp(self):
        super(SampleContainerTestCase,self).setUp()
        self.independent = FieldContainer(0.3*numpy.linspace(0,1,self.testData.shape[0]),
                                          longname='independent variable',
                                          shortname='x',
                                          unit = Quantity('1 mg'),
                                          attributes = copy.copy(self.attributes).update({'independent':True}))
        self.dependent = FieldContainer(9.81*self.independent.data,
                                        dimensions=[self.independent],
                                        longname='dependent variable',
                                        shortname='f',
                                        unit = Quantity('9.81 nN'),
                                        attributes = copy.copy(self.attributes).update({'independent':False}))
        self.sample = SampleContainer([self.dependent,self.field],longname='Sample',shortname='X',
                                      attributes = copy.copy(self.attributes).update({'isSample':'It seems so.'}))
        self.sample.seal()

    def testSaveRestore(self):
        self.eln.createGroup(self.eln.root,'testSaveRestoreSample')
        saveSample(self.eln,self.eln.root.testSaveRestoreSample,self.sample)
        restoredSample = loadSample(self.eln,self.eln.root.testSaveRestoreSample)
        self.assertEqual(restoredSample,self.sample)

class SampleContainerInSampleContainerTestCase(SampleContainerTestCase):
    def setUp(self):
        super(SampleContainerInSampleContainerTestCase,self).setUp()
        sample1 = SampleContainer([self.dependent,self.field],longname='First Sample',shortname='X',
                                      attributes = copy.copy(self.attributes).update({'isSample':'It seems so.'}))
        self.independent2 = FieldContainer(0.3*numpy.linspace(0,1,self.testData.shape[0]*10),
                                          longname='independent variable',
                                          shortname='x',
                                          unit = Quantity('1 mg'),
                                          attributes = copy.copy(self.attributes).update({'independent':True}))
        self.dependent2 = FieldContainer(9.81*self.independent2.data,
                                        dimensions=[self.independent2],
                                        longname='dependent variable',
                                        shortname='f',
                                        unit = Quantity('9.81 nN'),
                                        attributes = copy.copy(self.attributes).update({'independent':False}))

        sample2 = SampleContainer([self.dependent2,self.independent2],longname='Second Sample',shortname='Y',
                                      attributes = copy.copy(self.attributes).update({'sample Nr.': 2}))
        self.sample = SampleContainer([sample1,sample2],longname='SampleContainer with Samples',shortname='(X,Y)',
                                      attributes = copy.copy(self.attributes).update({'isSample':'It seems so.'}))
        self.sample.seal()

class ExecutionOrderTestCase(unittest.TestCase):
    def setUp(self):
        import tempfile
        self.path = tempfile.mktemp()
        self.orders = [({'sock1' : 'emd5://foo', 'sock2' : 'emd5://bar'}, 'baz'),
                       ({'sockA' : 'emd5://quux', 'sockB' : 'emd5://froz'}, 'pink')]

    def testSaveAndLoad(self):
        h5 = tables.openFile(self.path, mode='w')
        for order in self.orders:
            saveExecutionOrder(h5, order)
        h5.close()

        h5 = tables.openFile(self.path)
        orders = loadExecutionOrders(h5)
        h5.close()
        self.assertEquals(sorted(orders), sorted(self.orders))

if __name__ == "__main__":
    import sys
    if len(sys.argv) == 1:
        unittest.main()
    else:
        suite = unittest.TestLoader().loadTestsFromTestCase(eval(sys.argv[1:][0]))
        unittest.TextTestRunner().run(suite)



# -*- coding: utf-8 -*-

# Copyright (c) 2007-2008, Rectorate of the University of Freiburg
# Copyright (c) 2009, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import numpy

def fixedPoints(lambdaVec,kappa1=0.0):
    """Compute fixed points x0 and slope f'(x0) of f(x)=-lambda*x+x**3-kappa1."""
    #Compute constants, which do not depend on parameter lambda
    oneIsqrt3 = numpy.complex(1.0,numpy.sqrt(3.0))
    oneIMsqrt3 = numpy.complex(1.0,-numpy.sqrt(3.0))
    lambdaC = (float(abs(kappa1))/2)**(2.0/3.0)
    umCounter = (2.0/3.0)**(1.0/3.0)
    umDenominator = 2.0**(1.0/3.0)*3.0**(2.0/3.0)
    ulrDenominator = 2.0**(2.0/3.0)*3.0**(1.0/3.0)
    #Intialise lists of results
    mask = []
    x0 = []
    slope=[]
    #Loop over parameter lambda
    for lamb in lambdaVec:
        #Define function returning the slope for a given set of fixed points
        def computeSlope(u0):
            return -lamb + 3.0*u0**2
        #Compute constant, which does depend on parameter lambda
        R = (numpy.complex(-9.0,0.0)*kappa1+numpy.sqrt(3)*
             numpy.sqrt(numpy.complex(27.0,0.0)*kappa1**2-
                        numpy.complex(4.0,0.0)*lamb**3))**(1.0/3.0)
        #Compute fixed points of function f(x)=-lambda*x+x**3-kappa1
        um = -umCounter*lamb/R-R/umDenominator
        ul = oneIsqrt3*lamb/(ulrDenominator*R)+oneIMsqrt3*R/(2.0*umDenominator)
        ur = oneIMsqrt3*lamb/(ulrDenominator*R)+oneIsqrt3*R/(2.0*umDenominator)
        u0 = numpy.array([um,ul,ur])
        #Only real valued results reflect existing fixed points
        u01Pos = numpy.where(numpy.abs(numpy.imag(u0))<=1E-9,True,False)
        u0Real = numpy.real(u0[u01Pos])
        #Distuinguish between sub- and supercritical parameter ranges
        if len(u0Real)==1: #subcritical
            x0.append([u0Real[0],numpy.NaN,numpy.NaN])
            mask.append(numpy.array(map(lambda x: not x,[True,False,False])))
            slope.append(numpy.array([computeSlope(u0Real[0]),numpy.NaN,numpy.NaN]))
        else:#supercritical
            u0Sorted = numpy.sort(u0Real)
            x0.append(u0Sorted)
            mask.append(numpy.array(map(lambda x: not x,[True,True,True])))
            slope.append(numpy.array(map(computeSlope,u0Sorted)))
    return tuple(map(numpy.vstack, [x0,slope,mask]))
    #return x0,slope,mask



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2008, Rectorate of the University of Freiburg
# Copyright (c) 2009, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""


import copy
import pylab
import numpy
import pyphant.core.Connectors
from pyphant.core import DataContainer
from pyphant.wxgui2.DataVisReg import DataVisReg
from pyphant.quantities import isQuantity

class Chart(object):
    name='General chart'
    def __init__(self, dataContainer,show=True):
        self.dataContainer = dataContainer
        self.data = []
        self.prepare()
        for d in self.data:
            self.draw(**d)
        if show:
            self.finalize()

    def draw(self, **kwargs):
        pass

    def prepare(self):
        pylab.ioff()
        self.figure = pylab.figure()
        if isinstance(self.dataContainer.data[0], DataContainer.FieldContainer):
            ref = self.dataContainer.data[0]
            self.addDataContainer(ref)
            for fc in self.dataContainer.data[1:]:
                fc = fc.inUnitsOf(ref)
                fc.dimensions[-1] = fc.dimensions[-1].inUnitsOf(ref.dimensions[-1])
                self.addDataContainer(fc)
            pylab.xlabel(self.dataContainer.data[0].dimensions[-1].shortlabel)
            pylab.ylabel(self.dataContainer.data[0].label)
        else:
            self.addDataContainer(self.dataContainer)
            pylab.xlabel(self.dataContainer.dimensions[-1].shortlabel)
            pylab.ylabel(self.dataContainer.label)
        try:
            pylab.title(self.dataContainer.attributes['title'])
        except KeyError,TypeError:
            pass
        self.xmin = min([numpy.nanmin(d['abscissa']) for d in self.data])
        self.xmax = max([numpy.nanmax(d['abscissa']) for d in self.data])
        self.ymin = min([numpy.nanmin(d['data']) for d in self.data])
        self.ymax = max([numpy.nanmax(d['data']) for d in self.data])

    def addDataContainer(self, dataContainer):
        dataDict = {}
        if len(dataContainer.data.shape)==1:
            dataDict['data'] = dataContainer.data[numpy.newaxis,:]
            if dataContainer.error != None:
                dataDict['error'] = dataContainer.error[numpy.newaxis,:]
            else:
                dataDict['error'] = None
            if dataContainer.mask != None:
                dataDict['mask'] = dataContainer.mask[numpy.newaxis,:]
                dataDict['maskedData'] = numpy.ma.array(dataDict['data'], mask=dataDict['mask'])
            else:
                dataDict['mask'] = None
                dataDict['maskedData'] = numpy.ma.array(dataDict['data'])
        else:
            dataDict['data'] = dataContainer.data
            dataDict['error'] = dataContainer.error
            dataDict['mask'] = dataContainer.mask
            if dataDict['mask'] != None:
                dataDict['maskedData'] = numpy.ma.array(dataDict['data'], mask=dataDict['mask'])
            else:
                dataDict['maskedData'] = numpy.ma.array(dataDict['data'])
        dataDict['abscissa'] = dataContainer.dimensions[-1].data
        self.data.append(dataDict)


    def finalize(self):
        pylab.ion()
        pylab.show()


class BarChart(Chart):
    name = u"Bar chart"

    def draw(self, abscissa, data, maskedData, error, **kwargs):
        width = numpy.min(numpy.diff(abscissa))
        ymin = 0
        if self.ymin < ymin:
            ymin = self.ymin * 1.03
        if error == None:
            for ordinate in maskedData:
                pylab.bar(abscissa, ordinate, capsize=0,
                          width=width)
        else:
            for i in xrange(data.shape[0]):
                line = pylab.bar(abscissa, maskedData[i],
                                 yerr=error[i],
                                 width=width)
        xextent = (self.xmax-self.xmin)*0.03
        pylab.axis([self.xmin-xextent, self.xmax+width+xextent, ymin, self.ymax*1.03])


class LineChart(Chart):
    name = u"Line chart"
    linestyle = '-'

    def draw(self, abscissa, data, maskedData, error, **kwargs):
        if error == None:
            for ordinate in maskedData:
                pylab.plot(abscissa, ordinate, self.linestyle)
        else:
            for i in xrange(data.shape[0]):
                line = pylab.plot(abscissa, maskedData[i], self.linestyle)
                color = [l._color for l in line]
                for direction in [-1.,1.]:
                    ordinate = maskedData[i] + direction*error[i]
                    line = pylab.plot(abscissa, ordinate, linestyle=':')
                    for j in xrange(len(color)):
                        line[j]._color = color[j]
        pylab.axis([self.xmin, self.xmax, self.ymin, self.ymax])

class ScatterPlot(LineChart):
    name = u"Scatter Plot"
    linestyle = 'o'


DataVisReg.getInstance().registerVisualizer(pyphant.core.Connectors.TYPE_IMAGE, BarChart)
DataVisReg.getInstance().registerVisualizer(pyphant.core.Connectors.TYPE_IMAGE, LineChart)
DataVisReg.getInstance().registerVisualizer(pyphant.core.Connectors.TYPE_IMAGE, ScatterPlot)




# -*- coding: utf-8 -*-

# Copyright (c) 2006-2008, Rectorate of the University of Freiburg
# Copyright (c) 2009, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""


from pyphant.core import (Connectors, DataContainer)
from pyphant.wxgui2.DataVisReg import DataVisReg
import os
import wx
import csv

from scipy.io import write_array
from pyphant.quantities import isQuantity
import scipy

class ExternalDAT(object):
    name='Export to ASCII file'
    def __init__(self, dataContainer):
        self.dataContainer = dataContainer
        self.execute()

    def execute(self):
        dialog = wx.FileDialog(None,message='Choose file for saving the data', defaultDir=os.getcwd(),
                              style=wx.SAVE | wx.OVERWRITE_PROMPT,
                               wildcard = "Comma separated values (*.csv)|*.csv|Plain text (*.dat)|*.dat")
        if dialog.ShowModal() == wx.ID_OK:
            path = dialog.GetPath()
            print "Selected:",path
        else:
            print "Nothing was selected."
        dialog.Destroy()
        hash, uriType = DataContainer.parseId(self.dataContainer.id)
        if uriType == u"field":
            self.saveField(path)
        elif uriType == u"sample":
            self.saveSample(path)

    def saveField(self, path):
        if not isQuantity(self.dataContainer.unit):
            self.ordinate = self.dataContainer.data*self.dataContainer.unit
        else:
            self.ordinate = self.dataContainer.data*self.dataContainer.unit.value
        self.abscissa = self.dataContainer.dimensions[0].data
        outData = scipy.transpose(scipy.array([self.abscissa,self.ordinate]))
        if path[-3:] == 'csv':
            outFile = file(path,'wb')
            csvWriter = csv.writer(outFile,dialect='excel')
            csvWriter.writerow([self.dataContainer.dimensions[0].label, self.dataContainer.label])
            csvWriter.writerows(outData.tolist())
        else:
            outFile = file(path,'w')
            outFile.write(str([self.dataContainer.dimensions[0].label, self.dataContainer.label])+"\n")
            write_array(outFile,outData)
        outFile.close()

    def saveSample(self, path):
        outData = self.dataContainer.data

        if path[-3:] == 'csv':
            outFile = file(path,'wb')
            csvWriter = csv.writer(outFile,dialect='excel')
            #csvWriter.writerow([self.dataContainer.dimensions[0].label, self.dataContainer.label])
            csvWriter.writerows(outData.tolist())
        else:
            outFile = file(path,'w')
            #outFile.write(str([self.dataContainer.dimensions[0].label, self.dataContainer.label])+"\n")
            write_array(outFile,outData.tolist())
        outFile.close()


DataVisReg.getInstance().registerVisualizer(Connectors.TYPE_ARRAY, ExternalDAT)

















# -*- coding: utf-8 -*-

# Copyright (c) 2006-2008, Rectorate of the University of Freiburg
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""


import Connectors, EventDispatcher

def createParam(worker, paramName, displayName, values, subtype=None):
    if isinstance(values, list):
        return SelectionParam(worker, paramName, displayName,
                              values, subtype)
    else:
        return Param(worker, paramName, displayName, values, subtype)


class ParamChangeRequested(object):
    def __init__(self, param, oldValue, newValue):
        self.param = param
        self.oldValue = oldValue
        self.newValue = newValue


class ParamChanged(object):
    def __init__(self, param, oldValue, newValue):
        self.param = param
        self.oldValue = oldValue
        self.newValue = newValue


class ParamOverridden(object):
    def __init__(self, param, oldValue, newValue):
        self.param = param
        self.oldValue = oldValue
        self.newValue = newValue


class VetoParamChange(ValueError):
    def __init__(self, paramChangeEvent):
        ValueError.__init__(self, "Veto: change %s from %s to %s." %
                            (paramChangeEvent.param.name,
                             paramChangeEvent.oldValue,
                             paramChangeEvent.newValue))
        self.paramChangeEvent = paramChangeEvent


class Param(Connectors.Socket):
    def __getValue(self):
        if self.isFull():
            return self.getResult()
        else:
            return self._value

    def overrideValue(self, value):
        oldValue = self._value
        self._value = value
        self._eventDispatcher.dispatchEvent(
            ParamOverridden(self, oldValue, value))

    def __setValue(self, value):
        oldValue = self.value
        if oldValue == value:
            return
        self._eventDispatcher.dispatchEvent(
            ParamChangeRequested(self, oldValue, value)
            )
        self._value = value
        self._eventDispatcher.dispatchEvent(
            ParamChanged(self, oldValue, value)
            )
        self.invalidate()

    value = property(__getValue, __setValue)

    def __init__(self, worker, name, displayName, value, subtype=None):
        Connectors.Socket.__init__(self, worker, name, type(value))
        self.isExternal = False
        self.valueType = type(value)
        self.displayName = displayName
        self._value = value
        self.subtype = subtype
        self._eventDispatcher = EventDispatcher.EventDispatcher()

    def registerListener(self, vetoer, eventType):
        self._eventDispatcher.registerListener(vetoer, eventType)

    def unregisterListener(self, vetoer, eventType):
        self._eventDispatcher.unregisterListener(vetoer, eventType)

    #pickle
    def __getstate__(self):
        import copy
        pdict = copy.copy(self.__dict__)
        pdict['_eventDispatcher'] = EventDispatcher.EventDispatcher()
        return pdict


class SelectionParam(Param):
    def __init__(self, worker, name, displayName, values, subtype=None):
        Param.__init__(self, worker, name, displayName, values[0], subtype)
        self.valueType = type(values)
        self._possibleValues = values

    def getPossibleValues(self):
        return self._possibleValues

    def __getPossibleValues(self):
        return self.getPossibleValues()

    def setPossibleValues(self, values):
        self._possibleValues = values

    def __setPossibleValues(self, values):
        oldValues = self.possibleValues
        if oldValues == values:
            return
        self.setPossibleValues(values)
        self.invalidate()

    possibleValues = property(__getPossibleValues, __setPossibleValues)



# -*- coding: utf-8 -*-

# Copyright (c) 2009, Rectorate of the University of Freiburg
# Copyright (c) 2009-2011, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import unittest
from pyphant.quantities.ParseQuantities import parseDateTime,str2unit
from pyphant.quantities import Quantity


class TestParseDateTime(unittest.TestCase):
    """
    >>>parseDateTime('2004-08-21 12:00:00+-12hr')
    (Quantity(731814.5, 'd'), Quantity(0.5, 'd'))
    >>>parseDateTime('2004-08-21 12:00:00')
    (Quantity(731814.5, 'd'), None)
    """

    def testWithError(self):
        self.assertEqual(parseDateTime('2004-08-21 12:00:00+-12hr'),
                         (Quantity(731814.5, 'd'), Quantity(0.5, 'd'))
                         )

    def testWithErrorOldDeprecatedAbbreviation(self):
        self.assertEqual(parseDateTime('2004-08-21 12:00:00+-12h'),
                         (Quantity(731814.5, 'd'), Quantity(0.5, 'd'))
                         )


class TestStr2unit(unittest.TestCase):
    """Test the correct conversion of strings to quantities or floats."""

    def setUp(self):
        self.inputDict = {'complexJ':'1.0j', 'Joule':'1.0J'}

    def testSimpleQuantity(self):
        """
        The the conversion of a simple textual quantity specification
        to a quantity object.
        """
        expected = Quantity('1V')
        result = str2unit('1V')
        self.assertEqual(expected, result)

    def testComplexNumber(self):
        """
        Complex numbers have to be denoted by small 'j',
        in oder to discriminate them from Joule.
        """
        result = str2unit(self.inputDict['complexJ'])
        self.assertEqual(result, complex(self.inputDict['complexJ']))

    def testJouleValue(self):
        """Physical quantities with unit Joule are indicated by 'J'."""
        result = str2unit(self.inputDict['Joule'])
        self.assertEqual(result, Quantity(self.inputDict['Joule']))

    def testHourPlanck(self):
        """
        In FMF 1.0 unit 'h' denotes hours,
        while in FMF 1.1 'h' denotes the Planck constant.
        """
        result = str2unit('1h')
        self.assertEqual(result, Quantity('6.62606896e-34 J*s'))
        result = str2unit('1h', FMFversion='1.0')
        self.assertEqual(result, Quantity('3600s'))

    def testFloatAccuracy(self):
        result = str2unit('16.8 mm', FMFversion='1.0')
        diff = result - Quantity('16.8 mm')
        self.assertEqual(abs(diff.value) < 2e-14,True)
        result = str2unit('16.8 mm', FMFversion='1.0')
        diff = Quantity('16.8 mm')
        self.assertEqual(result, diff)

    def testGravitationalConstant(self):
        result = str2unit("1 Grav", FMFversion="1.0")
        self.assertEqual(result, str2unit('6.67259e-11 m**3/kg/s**2'))


if __name__ == "__main__":
    import sys
    if len(sys.argv) == 1:
        unittest.main()
    else:
        suite = unittest.TestLoader().loadTestsFromTestCase(
            eval(sys.argv[1:][0]))
        unittest.TextTestRunner().run(suite)



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2008, Rectorate of the University of Freiburg
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""Provides unittest classes
"""


import unittest
from pyphant.core.KnowledgeManager import (KnowledgeManager,
                                           CACHE_MAX_SIZE,
                                           CACHE_MAX_NUMBER)
import pyphant.core.PyTablesPersister as ptp
from pyphant.core.DataContainer import (FieldContainer, SampleContainer)
import numpy as N
import tables
import urllib
import tempfile
import os
import logging
from time import time
import random


class KnowledgeManagerTestCase(unittest.TestCase):
    def setUp(self):
        a = N.array([0, 1, 2, 3])
        self._fc = FieldContainer(a)
        self._fc.seal()

    def testGetLocalFile(self):
        h5fileid, h5name = tempfile.mkstemp(suffix='.h5',prefix='test-')
        os.close(h5fileid)
        h5 = tables.openFile(h5name,'w')
        resultsGroup = h5.createGroup("/", "results")
        ptp.saveResult(self._fc, h5)
        h5.close()
        km = KnowledgeManager.getInstance()
        from urllib import pathname2url
        url = pathname2url(h5name)
        if not url.startswith('///'):
            url = '//' + url
        url = 'file:' + url
        km.registerURL(url, temporary=True)
        km_fc = km.getDataContainer(self._fc.id)
        self.assertEqual(self._fc, km_fc)
        os.remove(h5name)

    def testGetHTTPFile(self):
        host = "pyphant.sourceforge.net"
        remote_dir = ""
        url = "http://" + host + remote_dir + "/knowledgemanager-http-test.h5"
        # Get remote file and load DataContainer
        filename, headers = urllib.urlretrieve(url)
        h5 = tables.openFile(filename, 'r')
        for g in h5.walkGroups("/results"):
            if (len(g._v_attrs.TITLE)>0) \
                    and (r"\Psi" in g._v_attrs.shortname):
                http_fc = ptp.loadField(h5,g)
        h5.close()
        km = KnowledgeManager.getInstance()
        km.registerURL(url, temporary=True)
        km_fc = km.getDataContainer(http_fc.id)
        self.assertEqual(http_fc, km_fc)
        os.remove(filename)

    def testGetDataContainer(self):
        km = KnowledgeManager.getInstance()
        km.registerDataContainer(self._fc, temporary=True)
        km_fc = km.getDataContainer(self._fc.id)
        self.assertEqual(self._fc, km_fc)

    def testSCwithSCColumn(self):
        fc_child1 = FieldContainer(longname='fc_child1', data=N.ones((10, 10)))
        fc_child2 = FieldContainer(longname='fc_child2', data=N.ones((20, 20)))
        sc_child = SampleContainer(longname='sc_child', columns=[fc_child1])
        sc_parent = SampleContainer(longname='sc_parent', columns=[sc_child,
                                                                   fc_child2])
        sc_parent.seal()
        km = KnowledgeManager.getInstance()
        km.registerDataContainer(sc_parent, temporary=True)
        lnlist = km.search(['longname'], {'col_of':{'longname':'sc_parent'}})
        lnlist = [entry[0] for entry in lnlist]
        assert len(lnlist) == 2
        assert 'fc_child2' in lnlist
        assert 'sc_child' in lnlist

    def testExceptions(self):
        km = KnowledgeManager.getInstance()
        #TODO:
        #invalid id
        #DataContainer not sealed
        #Local file not readable
        #Register empty hdf

    def testRegisterFMF(self):
        km = KnowledgeManager.getInstance()
        fileid, filename = tempfile.mkstemp(suffix='.fmf', prefix='test-')
        os.close(fileid)
        handler = open(filename, 'w')
        fmfstring = """; -*- fmf-version: 1.0 -*-
[*reference]
title: Knowledge Manager FMF Test
creator: Alexander Held
created: 2009-05-25 08:45:00+02:00
place: Uni Freiburg
[*data definitions]
voltage: V [V]
current: I(V) [A]
[*data]
1.0\t0.5
2.0\t1.0
3.0\t1.5
"""
        handler.write(fmfstring)
        handler.close()
        dc_id = km.registerFMF(filename, temporary=True)
        os.remove(filename)
        km.getDataContainer(dc_id)

    def testCache(self):
        print "Preparing FCs for cache test (cache size: %d MB)..."\
              % (CACHE_MAX_SIZE / 1024 / 1024)
        km = KnowledgeManager.getInstance()
        sizes = [(20, ), (10, 10, 10), (200, 200), (700, 700), (2000, 2000)]
        byte_sizes = [reduce(lambda x, y:x * y, size) * 8 for size in sizes]
        ids = []
        rand_id_pool = []
        assert_dict = {}
        for num in xrange(2 * 20):
            fc = FieldContainer(N.ones((500, 500)))
            fc.data.flat[0] = num
            fc.seal()
            km.registerDataContainer(fc, temporary=True)
            rand_id_pool.append(fc.id)
            assert_dict[fc.id] = num
        for size in sizes:
            fc = FieldContainer(N.ones(size))
            fc.seal()
            km.registerDataContainer(fc, temporary=True)
            ids.append(fc.id)
        for id, bytes in zip(ids, byte_sizes):
            uc_acc_time = 0.0
            c_acc_time = 0.0
            reps = 30
            for rep in xrange(reps):
                t1 = time()
                km.getDataContainer(id, use_cache=False)
                t2 = time()
                uc_acc_time += t2 - t1
            km._cache = []
            km._cache_size = 0
            for rep in xrange(reps):
                t1 = time()
                km.getDataContainer(id)
                t2 = time()
                c_acc_time += t2 - t1
            uc_avr = 1000.0 * uc_acc_time / reps
            c_avr = 1000.0 * c_acc_time / reps
            print "Avr. access time for %0.2f kB sequential read: "\
                  "%0.3f ms unchached, %0.3f ms cached" % (float(bytes) / 1024,
                                                           uc_avr, c_avr)
        km._cache = []
        km._cache_size = 0
        rand_ids = []
        reps = 500
        import pyphant.core.KnowledgeManager
        pyphant.core.KnowledgeManager.CACHE_MAX_NUMBER = 20
        for run in xrange(reps):
            rand_ids.append(rand_id_pool[
                random.randint(0, len(rand_id_pool) - 1)])
        uc_acc_time = 0.0
        c_acc_time = 0.0
        for id in rand_ids:
            t1 = time()
            fc = km.getDataContainer(id, use_cache=False)
            t2 = time()
            uc_acc_time += t2 - t1
        for id in rand_ids:
            t1 = time()
            fc = km.getDataContainer(id)
            t2 = time()
            c_acc_time += t2 - t1
            assert km._cache_size >= 0
            assert km._cache_size <= CACHE_MAX_SIZE
            assert len(km._cache) <= CACHE_MAX_NUMBER
            assert assert_dict[id] == fc.data.flat[0]
        uc_avr = 1000.0 * uc_acc_time / reps
        c_avr = 1000.0 * c_acc_time / reps
        bytes = float(500 * 500 * 8) / 1024
        print "Avr. access time for %0.2f kB random read: "\
              "%0.3f ms unchached, %0.3f ms cached" % (bytes, uc_avr, c_avr)
        print "------ End Cache Test ------"


if __name__ == "__main__":
    import sys
    logger = logging.getLogger('pyphant')
    hdlr = logging.StreamHandler(sys.stderr)
    formatter = logging.Formatter('[%(name)s|%(levelname)s] %(message)s')
    hdlr.setFormatter(formatter)
    logger.addHandler(hdlr)
    logger.setLevel(logging.DEBUG)
    if len(sys.argv) == 1:
        unittest.main()
    else:
        suite = unittest.TestLoader().loadTestsFromTestCase(
            eval(sys.argv[1:][0]))
        unittest.TextTestRunner().run(suite)



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2007, Rectorate of the University of Freiburg
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""

import logging
import singletonmixin
import pkg_resources


class ToolBoxInfo(object):
    def __init__(self, name):
        self.name = name
        self.workerInfos = []
        self._logger = logging.getLogger("pyphant")

    def sortWorkerInfos(self):
        self.workerInfos.sort(key=lambda x:x.name.lower())

    def __eq__(self, other):
        return self.name == other.name

    def __ne__(self, other):
        return self.name != other.name

    def addWorkerInfo(self, workerInfo):
        if not workerInfo in self.workerInfos:
            self.workerInfos.append(workerInfo)
            self._logger.info("Added worker %s from toolbox %s." \
                              % (workerInfo.name, self.name))


class WorkerRegistry(singletonmixin.Singleton):
    def __init__(self):
        self._toolBoxInfoList = []
        self._toolBoxInfoDict = {}
        self._logger = logging.getLogger("pyphant")
        self._dirty = True

    def registerWorker(self, workerInfo):
        tBI = ToolBoxInfo(workerInfo.toolBoxName)
        if not tBI in self._toolBoxInfoList:
            self._toolBoxInfoList.append(tBI)
            self._toolBoxInfoDict[tBI.name] = tBI
        self._toolBoxInfoDict[tBI.name].addWorkerInfo(workerInfo)

    def sortToolBoxInfos(self):
        for tBI in self._toolBoxInfoList:
            tBI.sortWorkerInfos()
        self._toolBoxInfoList.sort(key=lambda x: x.name.lower())

    def getToolBoxInfoList(self):
        if self._dirty:
            for worker in pkg_resources.iter_entry_points("pyphant.workers"):
                wm = worker.load()
                version = worker.dist.version
                for module in wm.workers:
                    try:
                        moduleName = worker.module_name + "." + module
                        self._logger.info("Trying to import " + moduleName)
                        exec 'import ' + moduleName
                        self._logger.info("Import module %s in version %s" \
                                          %(moduleName, version))
                    except ImportError, e:
                        self._logger.warning(
                            "worker archive " + worker.module_name \
                            + " contains invalid worker " + module \
                            + ": " + str(e))
            self.sortToolBoxInfos()
            self._dirty = False
        return self._toolBoxInfoList



# -*- coding: utf-8 -*-

# Copyright (c) 1998-2007, Konrad Hinsen <hinsen@cnrs-orleans.fr>
# Copyright (c) 2008-2009, Rectorate of the University of Freiburg
# Copyright (c) 2010, Andreas W. Liehr <liehr@users.sourceforge.net>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Quantities with units 

based on the module Scientific.Physics.PhysicalQuantities
written by Conrad Hinsen with contributions from Greg Ward. 
A comprehensive documentation of these units is given in 
Riede et al: On the Communication of Scientific Results: 
The Full-Metadata Format (http://arxiv.org/abs/0904.1299), 
which defines version 1.0 of the Full-Metadata Format.  

This module provides a data type that represents a physical
quantity together with its unit. It is possible to add and
subtract these quantities if the units are compatible, and
a quantity can be converted to another compatible unit.
Multiplication, subtraction, and raising to integer powers
is allowed without restriction, and the result will have
the correct unit. A quantity can be raised to a non-integer
power only if the result can be represented by integer powers
of the base units.

The values of physical constants are taken from the 1986
recommended values from CODATA. Other conversion factors
(e.g. for British units) come from various sources. I can't
guarantee for the correctness of all entries in the unit
table, so use this at your own risk.
"""

rc = { 'fetchCurrencyRates' : False }

class NumberDict(dict):

    """
    Dictionary storing numerical values

    Constructor: NumberDict()

    An instance of this class acts like an array of number with
    generalized (non-integer) indices. A value of zero is assumed
    for undefined entries. NumberDict instances support addition,
    and subtraction with other NumberDict instances, and multiplication
    and division by scalars.
    """

    def __getitem__(self, item):
        try:
            return dict.__getitem__(self, item)
        except KeyError:
            return 0

    def __coerce__(self, other):
        if type(other) == type({}):
            other = NumberDict(other)
        return self, other

    def __add__(self, other):
        sum_dict = NumberDict()
        for key in self.keys():
            sum_dict[key] = self[key]
        for key in other.keys():
            sum_dict[key] = sum_dict[key] + other[key]
        return sum_dict

    def __sub__(self, other):
        sum_dict = NumberDict()
        for key in self.keys():
            sum_dict[key] = self[key]
        for key in other.keys():
            sum_dict[key] = sum_dict[key] - other[key]
        return sum_dict

    def __mul__(self, other):
        new = NumberDict()
        for key in self.keys():
            new[key] = other*self[key]
        return new
    __rmul__ = __mul__

    def __div__(self, other):
        new = NumberDict()
        for key in self.keys():
            new[key] = self[key]/other
        return new

import numpy.oldnumeric
def int_sum(a, axis=0):
    return numpy.oldnumeric.add.reduce(a, axis)
def zeros_st(shape, other):
    return numpy.oldnumeric.zeros(shape, dtype=other.dtype)
from numpy import ndarray as array_type


import re, string


# Class definitions

class PhysicalQuantity:

    """
    Physical quantity with units

    PhysicalQuantity instances allow addition, subtraction,
    multiplication, and division with each other as well as
    multiplication, division, and exponentiation with numbers.
    Addition and subtraction check that the units of the two operands
    are compatible and return the result in the units of the first
    operand. A limited set of mathematical functions (from module
    Numeric) is applicable as well:

      - sqrt: equivalent to exponentiation with 0.5.

      - sin, cos, tan: applicable only to objects whose unit is
        compatible with 'rad'.

    See the documentation of the PhysicalQuantities module for a list
    of the available units.

    Here is an example on usage:

    >>> from PhysicalQuantities import PhysicalQuantity as p  # short hand
    >>> distance1 = p('10 m')
    >>> distance2 = p('10 km')
    >>> total = distance1 + distance2
    >>> total
    PhysicalQuantity(10010.0,'m')
    >>> total.convertToUnit('km')
    >>> total.getValue()
    10.01
    >>> total.getUnitName()
    'km'
    >>> total = total.inBaseUnits()
    >>> total
    PhysicalQuantity(10010.0,'m')
    >>>
    >>> t = p(314159., 's')
    >>> # convert to days, hours, minutes, and second:
    >>> t2 = t.inUnitsOf('d','h','min','s')
    >>> t2_print = ' '.join([str(i) for i in t2])
    >>> t2_print
    '3.0 d 15.0 h 15.0 min 59.0 s'
    >>>
    >>> e = p('2.7 Hartree*Nav')
    >>> e.convertToUnit('kcal/mol')
    >>> e
    PhysicalQuantity(1694.2757596034764,'kcal/mol')
    >>> e = e.inBaseUnits()
    >>> str(e)
    '7088849.77818 kg*m**2/s**2/mol'
    >>>
    >>> freeze = p('0 degC')
    >>> freeze = freeze.inUnitsOf ('degF')
    >>> str(freeze)
    '32.0 degF'
    >>>
    """

    def __init__(self, *args):
        """
        There are two constructor calling patterns:

            1. PhysicalQuantity(value, unit), where value is any number
            and unit is a string defining the unit

            2. PhysicalQuantity(value_with_unit), where value_with_unit
            is a string that contains both the value and the unit,
            i.e. '1.5 m/s'. This form is provided for more convenient
            interactive use.

        @param args: either (value, unit) or (value_with_unit,)
        @type args: (number, C{str}) or (C{str},)
        """
        if len(args) == 2:
            self.value = args[0]
            self.unit = _findUnit(args[1])
        else:
            s = string.strip(args[0])
            match = PhysicalQuantity._number.match(s)
            if match is None:
                raise TypeError('No number found')
            self.value = string.atof(match.group(0))
            self.unit = _findUnit(s[len(match.group(0)):])

    _number = re.compile('[+-]?[0-9]+(\\.[0-9]*)?([eE][+-]?[0-9]+)?')

    def __str__(self):
        return str(self.value) + ' ' + self.unit.name()

    def __repr__(self):
        return (self.__class__.__name__ + '(' + `self.value` + ',' +
                `self.unit.name()` + ')')

    def _sum(self, other, sign1, sign2):
        if not isPhysicalQuantity(other):
            raise TypeError('Incompatible types')
        new_value = sign1*self.value + \
                    sign2*other.value*other.unit.conversionFactorTo(self.unit)
        return self.__class__(new_value, self.unit)

    def __add__(self, other):
        return self._sum(other, 1, 1)

    __radd__ = __add__

    def __sub__(self, other):
        return self._sum(other, 1, -1)

    def __rsub__(self, other):
        return self._sum(other, -1, 1)

    def __cmp__(self, other):
        diff = self._sum(other, 1, -1)
        return cmp(diff.value, 0)

    def __mul__(self, other):
        if not isPhysicalQuantity(other):
            return self.__class__(self.value*other, self.unit)
        value = self.value*other.value
        unit = self.unit*other.unit
        if unit.isDimensionless():
            return value*unit.factor
        else:
            return self.__class__(value, unit)

    __rmul__ = __mul__

    def __div__(self, other):
        if not isPhysicalQuantity(other):
            return self.__class__(self.value/other, self.unit)
        value = self.value/other.value
        unit = self.unit/other.unit
        if unit.isDimensionless():
            return value*unit.factor
        else:
            return self.__class__(value, unit)

    def __rdiv__(self, other):
        if not isPhysicalQuantity(other):
            return self.__class__(other/self.value, pow(self.unit, -1))
        value = other.value/self.value
        unit = other.unit/self.unit
        if unit.isDimensionless():
            return value*unit.factor
        else:
            return self.__class__(value, unit)

    def __pow__(self, other):
        if isPhysicalQuantity(other):
            raise TypeError('Exponents must be dimensionless')
        return self.__class__(pow(self.value, other), pow(self.unit, other))

    def __rpow__(self, other):
        raise TypeError('Exponents must be dimensionless')

    def __abs__(self):
        return self.__class__(abs(self.value), self.unit)

    def __pos__(self):
        return self

    def __neg__(self):
        return self.__class__(-self.value, self.unit)

    def __nonzero__(self):
        return self.value != 0

    def convertToUnit(self, unit):
        """
        Change the unit and adjust the value such that
        the combination is equivalent to the original one. The new unit
        must be compatible with the previous unit of the object.

        @param unit: a unit
        @type unit: C{str}
        @raise TypeError: if the unit string is not a know unit or a
        unit incompatible with the current one
        """
        unit = _findUnit(unit)
        self.value = _convertValue (self.value, self.unit, unit)
        self.unit = unit

    def inUnitsOf(self, *units):
        """
        Express the quantity in different units. If one unit is
        specified, a new PhysicalQuantity object is returned that
        expresses the quantity in that unit. If several units
        are specified, the return value is a tuple of
        PhysicalObject instances with with one element per unit such
        that the sum of all quantities in the tuple equals the the
        original quantity and all the values except for the last one
        are integers. This is used to convert to irregular unit
        systems like hour/minute/second.

        @param units: one or several units
        @type units: C{str} or sequence of C{str}
        @returns: one or more physical quantities
        @rtype: L{PhysicalQuantity} or C{tuple} of L{PhysicalQuantity}
        @raises TypeError: if any of the specified units are not compatible
        with the original unit
        """
        units = map(_findUnit, units)
        if len(units) == 1:
            unit = units[0]
            value = _convertValue (self.value, self.unit, unit)
            return self.__class__(value, unit)
        else:
            units.sort()
            result = []
            value = self.value
            unit = self.unit
            for i in range(len(units)-1,-1,-1):
                value = value*unit.conversionFactorTo(units[i])
                if i == 0:
                    rounded = value
                else:
                    rounded = _round(value)
                result.append(self.__class__(rounded, units[i]))
                value = value - rounded
                unit = units[i]
            return tuple(result)

    # Contributed by Berthold Hoellmann
    def inBaseUnits(self):
        """
        @returns: the same quantity converted to base units,
        i.e. SI units in most cases
        @rtype: L{PhysicalQuantity}
        """
        new_value = self.value * self.unit.factor
        num = ''
        denom = ''
        for i in xrange(9):
            unit = _base_names[i]
            power = self.unit.powers[i]
            if power < 0:
                denom = denom + '/' + unit
                if power < -1:
                    denom = denom + '**' + str(-power)
            elif power > 0:
                num = num + '*' + unit
                if power > 1:
                    num = num + '**' + str(power)
        if len(num) == 0:
            num = '1'
        else:
            num = num[1:]
        return self.__class__(new_value, num + denom)

    def isCompatible (self, unit):
        """
        @param unit: a unit
        @type unit: C{str}
        @returns: C{True} if the specified unit is compatible with the
        one of the quantity
        @rtype: C{bool}
        """
        unit = _findUnit (unit)
        return self.unit.isCompatible (unit)

    def getValue(self):
        """Return value (float) of physical quantity (no unit)."""
        return self.value

    def getUnitName(self):
        """Return unit (string) of physical quantity."""
        return self.unit.name()

    def sqrt(self):
        return pow(self, 0.5)

    def sin(self):
        if self.unit.isAngle():
            return numpy.oldnumeric.sin(self.value * \
                             self.unit.conversionFactorTo(_unit_table['rad']))
        else:
            raise TypeError('Argument of sin must be an angle')

    def cos(self):
        if self.unit.isAngle():
            return numpy.oldnumeric.cos(self.value * \
                             self.unit.conversionFactorTo(_unit_table['rad']))
        else:
            raise TypeError('Argument of cos must be an angle')

    def tan(self):
        if self.unit.isAngle():
            return numpy.oldnumeric.tan(self.value * \
                             self.unit.conversionFactorTo(_unit_table['rad']))
        else:
            raise TypeError('Argument of tan must be an angle')


class PhysicalUnit:

    """
    Physical unit

    A physical unit is defined by a name (possibly composite), a scaling
    factor, and the exponentials of each of the SI base units that enter into
    it. Units can be multiplied, divided, and raised to integer powers.
    """

    def __init__(self, names, factor, powers, offset=0):
        """
        @param names: a dictionary mapping each name component to its
                      associated integer power (e.g. C{{'m': 1, 's': -1}})
                      for M{m/s}). As a shorthand, a string may be passed
                      which is assigned an implicit power 1.
        @type names: C{dict} or C{str}
        @param factor: a scaling factor
        @type factor: C{float}
        @param powers: the integer powers for each of the nine base units
        @type powers: C{list} of C{int}
        @param offset: an additive offset to the base unit (used only for
                       temperatures)
        @type offset: C{float}
        """
        if type(names) == type(''):
            self.names = NumberDict()
            self.names[names] = 1
        else:
            self.names = names
        self.factor = factor
        self.offset = offset
        self.powers = powers

    def __repr__(self):
        return '<PhysicalUnit ' + self.name() + '>'

    __str__ = __repr__

    def __cmp__(self, other):
        if self.powers != other.powers:
            raise TypeError('Incompatible units')
        return cmp(self.factor, other.factor)

    def __mul__(self, other):
        if self.offset != 0 or (isPhysicalUnit (other) and other.offset != 0):
            raise TypeError("cannot multiply units with non-zero offset")
        if isPhysicalUnit(other):
            return PhysicalUnit(self.names+other.names,
                                self.factor*other.factor,
                                map(lambda a,b: a+b,
                                    self.powers, other.powers))
        else:
            return PhysicalUnit(self.names+{str(other): 1},
                                self.factor*other,
                                self.powers,
                                self.offset * other)

    __rmul__ = __mul__

    def __div__(self, other):
        if self.offset != 0 or (isPhysicalUnit (other) and other.offset != 0):
            raise TypeError("cannot divide units with non-zero offset")
        if isPhysicalUnit(other):
            return PhysicalUnit(self.names-other.names,
                                self.factor/other.factor,
                                map(lambda a,b: a-b,
                                    self.powers, other.powers))
        else:
            return PhysicalUnit(self.names+{str(other): -1},
                                self.factor/other, self.powers)

    def __rdiv__(self, other):
        if self.offset != 0 or (isPhysicalUnit (other) and other.offset != 0):
            raise TypeError("cannot divide units with non-zero offset")
        if isPhysicalUnit(other):
            return PhysicalUnit(other.names-self.names,
                                other.factor/self.factor,
                                map(lambda a,b: a-b,
                                    other.powers, self.powers))
        else:
            return PhysicalUnit({str(other): 1}-self.names,
                                other/self.factor,
                                map(lambda x: -x, self.powers))

    def __pow__(self, other):
        if self.offset != 0:
            raise TypeError("cannot exponentiate units with non-zero offset")
        if isinstance(other, int):
            return PhysicalUnit(other*self.names, pow(self.factor, other),
                                map(lambda x,p=other: x*p, self.powers))
        if isinstance(other, float):
            inv_exp = 1./other
            rounded = int(numpy.oldnumeric.floor(inv_exp+0.5))
            if abs(inv_exp-rounded) < 1.e-10:
                if reduce(lambda a, b: a and b,
                          map(lambda x, e=rounded: x%e == 0, self.powers)):
                    f = pow(self.factor, other)
                    p = map(lambda x,p=rounded: x/p, self.powers)
                    if reduce(lambda a, b: a and b,
                              map(lambda x, e=rounded: x%e == 0,
                                  self.names.values())):
                        names = self.names/rounded
                    else:
                        names = NumberDict()
                        if f != 1.:
                            names[str(f)] = 1
                        for i in range(len(p)):
                            names[_base_names[i]] = p[i]
                    return PhysicalUnit(names, f, p)
                else:
                    raise TypeError('Illegal exponent')
        raise TypeError('Only integer and inverse integer exponents allowed')

    def conversionFactorTo(self, other):
        """
        @param other: another unit
        @type other: L{PhysicalUnit}
        @returns: the conversion factor from this unit to another unit
        @rtype: C{float}
        @raises TypeError: if the units are not compatible
        """
        if self.powers != other.powers:
            raise TypeError('Incompatible units')
        if self.offset != other.offset and self.factor != other.factor:
            raise TypeError(('Unit conversion (%s to %s) cannot be expressed ' +
                             'as a simple multiplicative factor') % \
                             (self.name(), other.name()))
        return self.factor/other.factor

    def conversionTupleTo(self, other): # added 1998/09/29 GPW
        """
        @param other: another unit
        @type other: L{PhysicalUnit}
        @returns: the conversion factor and offset from this unit to
                  another unit
        @rtype: (C{float}, C{float})
        @raises TypeError: if the units are not compatible
        """
        if self.powers != other.powers:
            raise TypeError('Incompatible units')

        # let (s1,d1) be the conversion tuple from 'self' to base units
        #   (ie. (x+d1)*s1 converts a value x from 'self' to base units,
        #   and (x/s1)-d1 converts x from base to 'self' units)
        # and (s2,d2) be the conversion tuple from 'other' to base units
        # then we want to compute the conversion tuple (S,D) from
        #   'self' to 'other' such that (x+D)*S converts x from 'self'
        #   units to 'other' units
        # the formula to convert x from 'self' to 'other' units via the
        #   base units is (by definition of the conversion tuples):
        #     ( ((x+d1)*s1) / s2 ) - d2
        #   = ( (x+d1) * s1/s2) - d2
        #   = ( (x+d1) * s1/s2 ) - (d2*s2/s1) * s1/s2
        #   = ( (x+d1) - (d1*s2/s1) ) * s1/s2
        #   = (x + d1 - d2*s2/s1) * s1/s2
        # thus, D = d1 - d2*s2/s1 and S = s1/s2
        factor = self.factor / other.factor
        offset = self.offset - (other.offset * other.factor / self.factor)
        return (factor, offset)

    def isCompatible (self, other):     # added 1998/10/01 GPW
        """
        @param other: another unit
        @type other: L{PhysicalUnit}
        @returns: C{True} if the units are compatible, i.e. if the powers of
                  the base units are the same
        @rtype: C{bool}
        """
        return self.powers == other.powers

    def isDimensionless(self):
        return not reduce(lambda a,b: a or b, self.powers)

    def isAngle(self):
        return self.powers[7] == 1 and \
               reduce(lambda a,b: a + b, self.powers) == 1

    def setName(self, name):
        self.names = NumberDict()
        self.names[name] = 1

    def name(self):
        num = ''
        denom = ''
        for unit in self.names.keys():
            power = self.names[unit]
            if power < 0:
                denom = denom + '/' + unit
                if power < -1:
                    denom = denom + '**' + str(-power)
            elif power > 0:
                num = num + '*' + unit
                if power > 1:
                    num = num + '**' + str(power)
        if len(num) == 0:
            num = '1'
        else:
            num = num[1:]
        return num + denom


# Type checks

def isPhysicalUnit(x):
    """
    @param x: an object
    @type x: any
    @returns: C{True} if x is a L{PhysicalUnit}
    @rtype: C{bool}
    """
    return hasattr(x, 'factor') and hasattr(x, 'powers')

def isPhysicalQuantity(x):
    """
    @param x: an object
    @type x: any
    @returns: C{True} if x is a L{PhysicalQuantity}
    @rtype: C{bool}
    """
    return hasattr(x, 'value') and hasattr(x, 'unit')


# Helper functions

def _findUnit(unit):
    if type(unit) == type(''):
        name = string.strip(unit)
        unit = eval(name, _unit_table)
        for cruft in ['__builtins__', '__args__']:
            try: del _unit_table[cruft]
            except: pass

    if not isPhysicalUnit(unit):
        raise TypeError(str(unit) + ' is not a unit')
    return unit

def _round(x):
    if numpy.oldnumeric.greater(x, 0.):
        return numpy.oldnumeric.floor(x)
    else:
        return numpy.oldnumeric.ceil(x)


def _convertValue (value, src_unit, target_unit):
    (factor, offset) = src_unit.conversionTupleTo(target_unit)
    return (value + offset) * factor


# SI unit definitions

_base_names = ['m', 'kg', 's', 'A', 'K', 'mol', 'cd', 'rad', 'sr','EUR']

_base_units = [('m',   PhysicalUnit('m',   1.,    [1,0,0,0,0,0,0,0,0,0])),
	       ('g',   PhysicalUnit('g',   0.001, [0,1,0,0,0,0,0,0,0,0])),
	       ('s',   PhysicalUnit('s',   1.,    [0,0,1,0,0,0,0,0,0,0])),
	       ('A',   PhysicalUnit('A',   1.,    [0,0,0,1,0,0,0,0,0,0])),
	       ('K',   PhysicalUnit('K',   1.,    [0,0,0,0,1,0,0,0,0,0])),
	       ('mol', PhysicalUnit('mol', 1.,    [0,0,0,0,0,1,0,0,0,0])),
	       ('cd',  PhysicalUnit('cd',  1.,    [0,0,0,0,0,0,1,0,0,0])),
	       ('rad', PhysicalUnit('rad', 1.,    [0,0,0,0,0,0,0,1,0,0])),
	       ('sr',  PhysicalUnit('sr',  1.,    [0,0,0,0,0,0,0,0,1,0])),
               ('EUR',  PhysicalUnit('EUR',  1.,  [0,0,0,0,0,0,0,0,0,1])),
	       ]

_prefixes = [('Y',  1.e24),
             ('Z',  1.e21),
             ('E',  1.e18),
             ('P',  1.e15),
             ('T',  1.e12),
             ('G',  1.e9),
             ('M',  1.e6),
             ('k',  1.e3),
             ('h',  1.e2),
             ('da', 1.e1),
             ('d',  1.e-1),
             ('c',  1.e-2),
             ('m',  1.e-3),
             ('mu', 1.e-6),
             ('n',  1.e-9),
             ('p',  1.e-12),
             ('f',  1.e-15),
             ('a',  1.e-18),
             ('z',  1.e-21),
             ('y',  1.e-24),
             ]

_unit_table = {}

for unit in _base_units:
    _unit_table[unit[0]] = unit[1]

_help = []

def _addUnit(name, unit, comment=''):
    if _unit_table.has_key(name):
	raise KeyError, 'Unit ' + name + ' already defined'
    if comment:
        _help.append((name, comment, unit))
    if type(unit) == type(''):
	unit = eval(unit, _unit_table)
        for cruft in ['__builtins__', '__args__']:
            try: del _unit_table[cruft]
            except: pass
    unit.setName(name)
    _unit_table[name] = unit

def _addPrefixed(unit):
    _help.append('Prefixed units for %s:' % unit)
    _prefixed_names = []
    if unit in ['EUR']:
        validPrefixes = filter(lambda prefix: prefix[1]>=1000,_prefixes) 
    else:
        validPrefixes = _prefixes
    for prefix in validPrefixes:
	name = prefix[0] + unit
	_addUnit(name, prefix[1]*_unit_table[unit])
        _prefixed_names.append(name)
    _help.append(', '.join(_prefixed_names))

# SI derived units; these automatically get prefixes
_help.append('SI derived units; these automatically get prefixes:\n' + \
     ', '.join([prefix + ' (%.0E)' % value for prefix, value in _prefixes]) + \
             '\n')


_unit_table['kg'] = PhysicalUnit('kg',   1., [0,1,0,0,0,0,0,0,0,0])

_addUnit('Hz', '1/s', 'Hertz')
_addUnit('N', 'm*kg/s**2', 'Newton')
_addUnit('Pa', 'N/m**2', 'Pascal')
_addUnit('J', 'N*m', 'Joule')
_addUnit('W', 'J/s', 'Watt')
_addUnit('C', 's*A', 'Coulomb')
_addUnit('V', 'W/A', 'Volt')
_addUnit('F', 'C/V', 'Farad')
_addUnit('ohm', 'V/A', 'Ohm')
_addUnit('S', 'A/V', 'Siemens')
_addUnit('Wb', 'V*s', 'Weber')
_addUnit('T', 'Wb/m**2', 'Tesla')
_addUnit('H', 'Wb/A', 'Henry')
_addUnit('lm', 'cd*sr', 'Lumen')
_addUnit('lx', 'lm/m**2', 'Lux')
_addUnit('Bq', '1/s', 'Becquerel')
_addUnit('Gy', 'J/kg', 'Gray')
_addUnit('Sv', 'J/kg', 'Sievert')

del _unit_table['kg']

for unit in _unit_table.keys():
    _addPrefixed(unit)

# Fundamental constants
_help.append('Fundamental constants:')

_unit_table['pi'] = numpy.oldnumeric.pi
_addUnit('c', '299792458.*m/s', 'speed of light')
_addUnit('mu0', '4.e-7*pi*N/A**2', 'permeability of vacuum')
_addUnit('eps0', '1/mu0/c**2', 'permittivity of vacuum')
_addUnit('Grav', '6.67259e-11*m**3/kg/s**2', 'gravitational constant')
_addUnit('hplanck', '6.6260755e-34*J*s', 'Planck constant')
_addUnit('hbar', 'hplanck/(2*pi)', 'Planck constant / 2pi')
_addUnit('e', '1.60217733e-19*C', 'elementary charge')
_addUnit('me', '9.1093897e-31*kg', 'electron mass')
_addUnit('mp', '1.6726231e-27*kg', 'proton mass')
_addUnit('Nav', '6.0221367e23/mol', 'Avogadro number')
_addUnit('k', '1.380658e-23*J/K', 'Boltzmann constant')

# Time units
_help.append('Time units:')

_addUnit('min', '60*s', 'minute')
_addUnit('h', '60*min', 'hour')
_addUnit('d', '24*h', 'day')
_addUnit('wk', '7*d', 'week')
_addUnit('yr', '365.25*d', 'year')

# Length units
_help.append('Length units:')

_addUnit('inch', '2.54*cm', 'inch')
_addUnit('ft', '12*inch', 'foot')
_addUnit('yd', '3*ft', 'yard')
_addUnit('mi', '5280.*ft', '(British) mile')
_addUnit('nmi', '1852.*m', 'Nautical mile')
_addUnit('Ang', '1.e-10*m', 'Angstrom')
_addUnit('lyr', 'c*yr', 'light year')
_addUnit('AU', '149597870691*m', 'astronomical unit')
_addUnit('pc', '3.08567758128E16*m','parsec')
_addUnit('Bohr', '4*pi*eps0*hbar**2/me/e**2', 'Bohr radius')

# Area units
_help.append('Area units:')

_addUnit('ha', '10000*m**2', 'hectare')
_addUnit('acres', 'mi**2/640', 'acre')
_addUnit('b', '1.e-28*m', 'barn')

# Volume units
_help.append('Volume units:')

_addUnit('l', 'dm**3', 'liter')
_addUnit('dl', '0.1*l', 'deci liter')
_addUnit('cl', '0.01*l', 'centi liter')
_addUnit('ml', '0.001*l', 'milli liter')
_addUnit('tsp', '4.92892159375*ml', 'teaspoon')
_addUnit('tbsp', '3*tsp', 'tablespoon')
_addUnit('floz', '2*tbsp', 'fluid ounce')
_addUnit('cup', '8*floz', 'cup')
_addUnit('pt', '16*floz', 'pint')
_addUnit('qt', '2*pt', 'quart')
_addUnit('galUS', '4*qt', 'US gallon')
_addUnit('galUK', '4.54609*l', 'British gallon')

# Mass units
_help.append('Mass units:')

_addUnit('amu', '1.6605402e-27*kg', 'atomic mass units')
_addUnit('oz', '28.349523125*g', 'ounce')
_addUnit('lb', '16*oz', 'pound')
_addUnit('ton', '2000*lb', 'ton')

# Concentration units
_help.append('Concentration units:')
_addUnit('M','mol/m**3','molar concentration')
_addUnit('mM','0.001*mol/m**3','millimolar')
_addUnit('muM','10**-6*mol/m**3','micromolar')

# Force units
_help.append('Force units:')

_addUnit('dyn', '1.e-5*N', 'dyne (cgs unit)')

# Energy units
_help.append('Energy units:')

_addUnit('erg', '1.e-7*J', 'erg (cgs unit)')
_addUnit('eV', 'e*V', 'electron volt')
_addUnit('Hartree', 'me*e**4/16/pi**2/eps0**2/hbar**2', 'Wavenumbers/inverse cm')
_addUnit('Ken', 'k*K', 'Kelvin as energy unit')
_addUnit('cal', '4.184*J', 'thermochemical calorie')
_addUnit('kcal', '1000*cal', 'thermochemical kilocalorie')
_addUnit('cali', '4.1868*J', 'international calorie')
_addUnit('kcali', '1000*cali', 'international kilocalorie')
_addUnit('Btu', '1055.05585262*J', 'British thermal unit')

_addPrefixed('eV')

# Power units
_help.append('Power units:')

_addUnit('hp', '745.7*W', 'horsepower')

# Pressure units
_help.append('Pressure units:')

_addUnit('bar', '1.e5*Pa', 'bar (cgs unit)')
_addUnit('dbar', '1.e4*Pa', 'dbar (cgs unit)')
_addUnit('mbar', '1.e2*Pa', 'mbar (cgs unit)')
_addUnit('atm', '101325.*Pa', 'standard atmosphere')
_addUnit('torr', 'atm/760', 'torr = mm of mercury')
_addUnit('psi', '6894.75729317*Pa', 'pounds per square inch')

# Angle units
_help.append('Angle units:')

_addUnit('deg', 'pi*rad/180', 'degrees')

_help.append('Temperature units:')
# Temperature units -- can't use the 'eval' trick that _addUnit provides
# for degC and degF because you can't add units
kelvin = _findUnit ('K')
_addUnit ('degR', '(5./9.)*K', 'degrees Rankine')
_addUnit ('degC', PhysicalUnit (None, 1.0, kelvin.powers, 273.15),
          'degrees Celcius')
_addUnit ('degF', PhysicalUnit (None, 5./9., kelvin.powers, 459.67),
          'degree Fahrenheit')
del kelvin

_help.append('Old European currencies:')
#Taken from http://www.xe.com/euro.php on 2008-11-05
_addUnit('ATS', 'EUR/13.7603' ,'Austria, Schilling')
_addUnit('BEF', 'EUR/40.3399' ,'Belgium, Franc')
_addUnit('CYP', 'EUR/0.585274','Cyprus, Pound')
_addUnit('DEM', 'EUR/1.95583' ,'Germany, Deutsche Mark')
_addUnit('ESP', 'EUR/166.386' ,'Spain, Peseta')
_addUnit('FIM', 'EUR/5.94573' ,'Finland, Markka')
_addUnit('FRF', 'EUR/5.94573' ,'France, Franc')
_addUnit('GRD', 'EUR/340.750' ,'Greece, Drachma')
_addUnit('IEP', 'EUR/0.787564','Ireland, Pound')
_addUnit('ITL', 'EUR/1936.27' ,'Italy, Lira')
_addUnit('LUF', 'EUR/40.3399' ,'Luxembourg, Franc')
_addUnit('MTL', 'EUR/0.429300','Malta, Lira')
_addUnit('NLG', 'EUR/2.20371' ,'The Netherlands, Guilder (also called Florin)')
_addUnit('PTE', 'EUR/200.482' ,'Portugal, Escudo')
_addUnit('SIT', 'EUR/239.640' ,'Slovenia, Tolar')
_addUnit('VAL', 'EUR/1936.27' ,'Vatican City, Lira')

#Get daily updated exchange rates
if rc['fetchCurrencyRates']:
    import urllib
    from xml.dom import minidom
    url = "http://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml"
    currencyNames={'USD':'US dollar'  ,     'JPY':'Japanese yen',
                   'BGN':'Bulgarian lev'  , 'CZK':'Czech koruna',
                   'DKK':'Danish krone'  ,  'EEK':'Estonian kroon',
                   'GBP':'Pound sterling'  ,'HUF':'Hungarian forint',
                   'LTL':'Lithuanian litas','LVL':'Latvian lats',
                   'PLN':'Polish zloty',    'RON':'New Romanian leu',
                   'SEK':'Swedish krona',   'SKK':'Slovak koruna',
                   'CHF':'Swiss franc',     'ISK':'Icelandic krona',
                   'NOK':'Norwegian krone', 'HRK':'Croatian kuna',
                   'RUB':'Russian rouble',  'TRY':'New Turkish lira',
                   'AUD':'Australian dollar','BRL':'Brasilian real',
                   'CAD':'Canadian dollar', 'CNY':'Chinese yuan renminbi',
                   'HKD':'Hong Kong dollar','IDR':'Indonesian rupiah',
                   'KRW':'South Korean won','MXN':'Mexican peso',
                   'MYR':'Malaysian ringgit','NZD':'New Zealand dollar',
                   'PHP':'Philippine peso', 'SGD':'Singapore dollar',
                   'THB':'Thai baht',       'ZAR':'South African rand'}
    try:
        doc = minidom.parseString(urllib.urlopen(url).read())
        elements = doc.documentElement.getElementsByTagName('Cube')
        for element in elements[2:]:
            currency = element.getAttribute('currency').encode('utf8')
            _addUnit(currency,
                     'EUR/%s' % element.getAttribute('rate').encode('utf8'),
                     currencyNames[currency])
        print "Added exchange rate of %s for %s." % (elements[1].getAttribute('time'),
                                                     [ i.getAttribute('currency').encode('utf8')
                                                       for i in elements[2:] ])
    except:
        print "WARNING: No daily exchange rates available."

def description():
    """Return a string describing all available units."""
    s = ''  # collector for description text
    for entry in _help:
        if isinstance(entry, basestring):
            # headline for new section
            s += '\n' + entry + '\n'
        elif isinstance(entry, tuple):
            name, comment, unit = entry
            s += '%-8s  %-26s %s\n' % (name, comment, unit)
        else:
            # impossible
            raise TypeError, 'wrong construction of _help list'
    return s

# add the description of the units to the module's doc string:
__doc__ += '\n' + description()

# Some demonstration code. Run with "python -i PhysicalQuantities.py"
# to have this available.

if __name__ == '__main__':

#    from Scientific.N import *
    l = PhysicalQuantity(10., 'm')
    big_l = PhysicalQuantity(10., 'km')
    print big_l + l
    t = PhysicalQuantity(314159., 's')
    print t.inUnitsOf('d','h','min','s')

    p = PhysicalQuantity # just a shorthand...

    e = p('2.7 Hartree*Nav')
    e.convertToUnit('kcal/mol')
    print e
    print e.inBaseUnits()

    freeze = p('0 degC')
    print freeze.inUnitsOf ('degF')

    euro = PhysicalQuantity('1 EUR')
    print euro.inUnitsOf('DEM')
    print euro.inUnitsOf('USD')

    euroSQM = PhysicalQuantity('19.99 EUR/m**2')
    print "%s=%s" % (euroSQM,euroSQM.inUnitsOf('EUR/cm**2'))



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2007, Rectorate of the University of Freiburg
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""


import Queue
import wx
from pyphant.core.Connectors import Computer

        

class ConfigureFrame(wx.Dialog):
    def __init__(self, parent, paramVisReg, worker):
        from PyphantDiagram import ProgressMeter
        self._paramVisReg=paramVisReg
        wx.Dialog.__init__(self, parent, -1, "Configure "+worker.getParam("name").value)
        self._paramDict={}
        progress = ProgressMeter("Acquiring Param data")
        exception_queue = Queue.Queue()
        computer = Computer(worker.refreshParams, exception_queue, subscriber=progress)
        computer.start()
        while (progress.percentage<100) and (computer.isAlive()):
            progress.update()
        progress.finishAllProcesses()
        progress.Destroy()
        if computer.isAlive():
            computer.join()
        pl = worker.getParamList()
        if len(pl) <= 0:
            self.Add(wx.StaticText(self, label="Nothing to be configured"))
        else:
            sizer = wx.FlexGridSizer(len(pl) + 2, 3, 5, 5)
            sizer.Add(wx.StaticText(self, label="Label"))
            sizer.Add(wx.StaticText(self, label="Value"))
            sizer.Add(wx.StaticText(self, label="is external"))
            for param in pl:
                sizer.Add(wx.StaticText(self, label=param.displayName))
                vis=self._paramVisReg.createVisualizerFor(self, param)
                sizer.Add(vis)
                checkBox=wx.CheckBox(self)
                checkBox.SetValue(param.isExternal)
                sizer.Add(checkBox)
                self._paramDict[vis]=(param,checkBox)
            sizer.Add(self.CreateButtonSizer(wx.OK | wx.CANCEL))
            self.SetSizer(sizer)
            sizer.Fit(self)

    def applyAll(self, event=None):
        for (vis,(param,checkBox)) in self._paramDict.items():
            try:
                param.value=vis.getValue()
                param.isExternal=checkBox.GetValue()
            except ValueError, (e):
                print "Caught a ValueError in ", vis, ":", e

